

package org.objectweb.petals.binding.axis2bc.engine;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.AddressingConstants;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.context.OperationContext;
import org.apache.axis2.context.ServiceContext;
import org.apache.axis2.context.ServiceGroupContext;
import org.apache.axis2.description.AxisOperation;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.HandlerDescription;
import org.apache.axis2.engine.AbstractDispatcher;
import org.apache.axis2.om.OMElement;
import org.apache.axis2.util.Utils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.objectweb.petals.binding.axis2bc.Axis2BC;
import org.objectweb.petals.binding.axis2bc.receiver.PetalsReceiver;
import org.objectweb.petals.binding.axis2bc.util.SourceHelper;

/**
 * Dispatcher based on the WS-Addressing properties.
 */
public class PetalsDispatcher extends AbstractDispatcher implements AddressingConstants {



	private static final long serialVersionUID = 1L;
	/**
	 * Field NAME
	 */
	public static final QName NAME = new QName("http://ws.apache.org/axis2/",
	"PetalsDispatcher");
	
	private Log log = LogFactory.getLog(Axis2BC.class);
	
	private long timeout = 2000 ;

	/*
	 * (non-Javadoc)
	 * @see org.apache.axis2.engine.AbstractDispatcher#findOperation(org.apache.axis2.description.AxisService, org.apache.axis2.context.MessageContext)
	 */
	public AxisOperation findOperation(AxisService service, MessageContext messageContext)
	throws AxisFault {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * @see org.apache.axis2.engine.AbstractDispatcher#findService(org.apache.axis2.context.MessageContext)
	 */
	public AxisService findService(MessageContext messageContext) throws AxisFault {

		QName operationName = null;
		String serviceName = null;
		boolean messageExchangeStatus = false;
		// Get Service name using endpoint
		EndpointReference toEPR = messageContext.getTo();

		log.info("Checking for Service using target endpoint address : " + toEPR.getAddress());

		String filePart = toEPR.getAddress();
		String[] values = Utils.parseRequestURLForServiceAndOperation(filePart);

		if (values[1] != null) {
			operationName = new QName(values[1]);
		}

		if (values[0] != null) {
			serviceName = values[0];
		}

		// Get operation name using SOAP body
		OMElement bodyFirstChild = messageContext.getEnvelope().getBody().getFirstElement();

		if (bodyFirstChild != null) {
			log.info("Checking for Operation using SOAP message body's first child's local name : "
					+ bodyFirstChild.getLocalName());
			operationName = new QName(bodyFirstChild.getLocalName());
		}

		// Check if this service and operation exist on the jbi container

		Source source = SourceHelper.createSource("Axis2 Petals dispatcher test message");
		MessageExchange msg = null;
		try{
			msg = Axis2BC.context.getDeliveryChannel().createExchangeFactory().createInOutExchange();
			NormalizedMessage nm = msg.createMessage();
			nm.setContent(source);
			((InOut)msg).setInMessage(nm);
			log.info("Petals Dispatcher : looking for service :"+serviceName+" operation:"+operationName.getLocalPart());
			msg.setService(new QName(serviceName));
			msg.setOperation(operationName);
			messageExchangeStatus = Axis2BC.context.getDeliveryChannel().sendSync(msg,timeout);
		}
		catch (MessagingException e){
			log.error("Petals Dispatcher : Requested service and/or operation not found/available");
			log.error(e.getMessage());
			return null;
		}
		

		if(!messageExchangeStatus)
			return null;
		else{
			if(!msg.getStatus().equals(ExchangeStatus.DONE)){
				log.error(msg.getError().getMessage());
				return null;
			}
		}
		
		// Service found, return the axisService and AxisOperation
		AxisService axisServ = messageContext.getConfigurationContext().getAxisConfiguration().getService(serviceName);

		PetalsReceiver receiver = new PetalsReceiver();

		if (axisServ == null ){

			AxisService service = Utils.createSimpleService(new QName(serviceName), receiver, "PetalsReceiver",	operationName);
			messageContext.getConfigurationContext().getAxisConfiguration().addService(service);
			return service;
		}
		else{
			if(axisServ.getOperation(operationName) == null){
				messageContext.getConfigurationContext().getAxisConfiguration().removeService(serviceName);
				AxisService service = Utils.createSimpleService(new QName(serviceName), receiver, "PetalsReceiver",operationName);
				messageContext.getConfigurationContext().getAxisConfiguration().addService(service);
				return service;
			}
			return axisServ;
		}
	}

	public void initDispatcher() {
		init(new HandlerDescription(NAME));
	}

	/**
	 * @param msgctx
	 * @throws org.apache.axis2.AxisFault
	 */
	public void invoke(MessageContext msgctx) throws AxisFault {

		// first check we can dispatch using the relates to
		if (msgctx.getRelatesTo() != null) {
			log.debug("Checking RelatesTo : " + msgctx.getRelatesTo());

			String relatesTo = msgctx.getRelatesTo().getValue();

			if ((relatesTo != null) || "".equals(relatesTo)) {
				OperationContext operationContext =
					msgctx.getConfigurationContext().getOperationContext(relatesTo);

				if (operationContext != null) {
					msgctx.setAxisOperation(operationContext.getAxisOperation());
					msgctx.setOperationContext(operationContext);
					msgctx.setServiceContext((ServiceContext) operationContext.getParent());
					msgctx.setAxisService(
							((ServiceContext) operationContext.getParent()).getAxisService());
					msgctx.getAxisOperation().registerOperationContext(msgctx, operationContext);
					msgctx.setServiceGroupContextId(
							((ServiceGroupContext) msgctx.getServiceContext().getParent()).getId());
				}
			}

			return;
		}
		super.invoke(msgctx);
	}
}
