package org.objectweb.petals.binding.axis2bc.receiver;

import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.AxisOperation;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.om.OMAttribute;
import org.apache.axis2.om.OMElement;
import org.apache.axis2.om.OMNamespace;
import org.apache.axis2.om.impl.llom.OMElementImpl;
import org.apache.axis2.receivers.AbstractInOutSyncMessageReceiver;
import org.apache.axis2.soap.SOAPBody;
import org.apache.axis2.soap.SOAPEnvelope;
import org.apache.axis2.soap.SOAPFactory;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.objectweb.petals.binding.axis2bc.Axis2BC;
import org.objectweb.petals.binding.axis2bc.util.SourceHelper;

/**
 * This is a MessageReceiver which is used on the client side to accept the
 * messages (response) that come to the client. This correlates the incoming message to
 * the related messages and makes a call to the appropriate callback.
 */
public class PetalsReceiver extends AbstractInOutSyncMessageReceiver{
	public static String SERVICE_NAME = "Helloworld";
	@SuppressWarnings("unused")

	private Log log = LogFactory.getLog(Axis2BC.class);

	public PetalsReceiver() {
		super();
	}


	/**
	 * @param inMessage MessageContext for incoming message
	 * @param outMessage MessageContect fot outgoing message
	 */
	public  void invokeBusinessLogic(MessageContext inMessage, MessageContext outMessage
	) throws AxisFault{
		log.debug("Petals Listenner : invokeBusinessLogic");

		AxisOperation axisOperation = inMessage.getOperationContext().getAxisOperation();

		AxisService axisService = inMessage.getServiceContext().getAxisService();

		Source source = SourceHelper.createSource(inMessage.getEnvelope().toString());

		MessageExchange msg = null;
		try{
			msg = Axis2BC.context.getDeliveryChannel().createExchangeFactory().createInOutExchange();


			NormalizedMessage nm = msg.createMessage();
			nm.setContent(source);
			((InOut)msg).setInMessage(nm);
			log.debug("Petals Receiver : querying service :"+axisService.getName()+" operation:"
					+axisOperation.getName().getLocalPart());
			msg.setService(new QName(axisService.getName()));

			msg.setOperation(axisOperation.getName());

			Axis2BC.context.getDeliveryChannel().sendSync(msg);
			
			NormalizedMessage nm2 = ((InOut)msg).getOutMessage();
			// Make the response

			SOAPFactory factory = getSOAPFactory(inMessage);
			OMNamespace ns = factory.createOMNamespace("http://soapenc/", "res");
			OMNamespace ns2 = factory.createOMNamespace("http://www.w3.org/1999/XMLSchema-instance","xsi");
			OMNamespace ns3 = factory.createOMNamespace("http://www.w3.org/1999/XMLSchema","xsd");
			SOAPEnvelope responseEnv = factory.createSOAPEnvelope();
			responseEnv.declareNamespace(ns2);
			responseEnv.declareNamespace(ns3);
			SOAPBody responseBody = factory.createSOAPBody(responseEnv);
			OMElement respBody = factory.createOMElement(axisService.getName()+"Response",ns);

			OMElement node = new OMElementImpl("return",ns);
			
			if(nm2 != null)
				node.setText(SourceHelper.createString(nm2.getContent()));
			else
				node.setText("Done");
			OMAttribute typeAttribute = factory.createOMAttribute("type", ns2,"xsd:string");
			node.addAttribute(typeAttribute);
			respBody.addChild(node);
			// respBody.setText("This is the response to your query");
			responseBody.addChild(respBody);
			outMessage.setEnvelope(responseEnv);
		}
		catch (MessagingException e){
			log.error("Petals Dispatcher : Requested service and/or operation not found/available");
			log.error(e.getMessage());
		}
		catch(Exception e){
			log.error("Petals Dispatcher : Requested service and/or operation not found/available");
			log.error(e.getMessage());
		}
	}
}
