/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 � EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: SourceHelper.java 94 2006-01-26 17:11:05Z rmarins $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.binding.axis2bc.util;

import java.io.ByteArrayInputStream;

import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

/**
 * Sample class used to manage the mapping between 
 * <code>javax.xml.transform.Source</code> from String.
 * the mapping is a quicly "string" -> "<content>string</content>" conversion. 
 * @author Adrien LOUIS - EBM WebSourcing
  */
public class SourceHelper {

    private static final String START_TAG = "<content>";
    private static final String END_TAG = "</content>";
    
    public static Source createContentSource(String msg)
    {
        return createSource(START_TAG+msg+END_TAG);
    }
    
    public static String createStringContent(Source s) throws Exception
	{
        String content = createString(s);
        int index = content.indexOf(START_TAG);
        String result = content.substring(index+START_TAG.length());
        return result.substring(0,result.indexOf(END_TAG));
    }
    
    
	public static Source createSource(String msg)
	{
		StreamSource source = new StreamSource();
		
		byte[] msgByte = msg.getBytes();
		
		ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
		
		source.setInputStream(in);
		
		return source;
	}
	
	public static String createString(Source s) throws Exception
	{
		if(s == null || !(s instanceof StreamSource) )
		{
			throw new Exception("source is null or is not a streamsource");
		}
		
		ByteArrayInputStream bio = (ByteArrayInputStream) ((StreamSource)s).getInputStream();
		
		if(bio.available() == 0)
		{
			throw new Exception("source is empty");
		}
		
		byte[] buff = new byte[bio.available()];
		
		bio.read(buff);
		
		String str = new String(buff);
		
		return str;
	}
}
