package org.objectweb.petals.engine.clock;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.objectweb.petals.engine.util.SourceHelper;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;

public class Clock implements Component, ComponentLifeCycle {
	
	private ComponentContext context;
	
	private DeliveryChannel channel;
	
	@SuppressWarnings("unused")
	private ServiceEndpoint endpointReference;
	
	private ClockListener listener;
	
	private static final String SERVICE = "ClockService";
    private static final String ENDPOINT = "ClockEndpoint";
	
	private Logger logger;
	
	private Map<String,TimeZonesMap> timeZoneMaps;
	
	public void init(ComponentContext context) throws JBIException 
	{
		this.context = context;
		
		logger = context.getLogger("",null);

		logger.log(Level.INFO,"");
		
		timeZoneMaps = new HashMap<String,TimeZonesMap>();
	}

	
	public void shutDown() throws JBIException 
	{
		this.listener.stopProcessing();
	}

	
	public void start() throws JBIException 
	{
		logger.log(Level.INFO,"");
		
		try 
		{
			this.channel = this.context.getDeliveryChannel();
			this.listener = new ClockListener(this.channel,logger);
		
			Thread listenerThread = new Thread(this.listener);
			listenerThread.start();
			
			endpointReference = this.context.activateEndpoint(new QName(SERVICE), ENDPOINT); 
		} 
		catch (MessagingException e) 
		{
			throw new JBIException(e);
		}

	}

	
	public void stop() throws JBIException {
		logger.log(Level.INFO,"");
		
		this.listener.stopProcessing();
	}
	
	
	public ComponentLifeCycle getLifeCycle() 
	{
		return this;
	}

	public Document getServiceDescription(ServiceEndpoint arg0) 
	{
		if(  new QName(SERVICE).equals(arg0.getServiceName()))
		{
			File wsdlFile = new File(context.getInstallRoot()+"/META-INF/ClockService.wsdl");
	
			return SourceHelper.createDocumentFromWSDL(wsdlFile);
		}
		return null;
	}

	public ServiceUnitManager getServiceUnitManager() 
	{
		return new ClockServiceUnitManager(this);
	}

	public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
			MessageExchange arg1) {
		logger.log(Level.INFO,"accept the exchange");
		return true;
	}

	public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
			MessageExchange arg1) {
		return false;
	}

	public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0) {
		ServiceEndpoint result = null;
		
		if (arg0 != null)
		{
			Node service = SourceHelper.findChild(arg0,"service",true);

			if (service != null)
			{
				String serviceRequested = SourceHelper.getAttributeValue(service,"name");
				
				if (SERVICE.equals(serviceRequested))
				{
					result = endpointReference; 
				}
			}
		}
		return result;
	}

	public ObjectName getExtensionMBeanName() {
		// TODO Auto-generated method stub
		return null;
	}
	
	protected TimeZonesMap getTimeZonesForArtifact(String artifactName){
		return timeZoneMaps.get(artifactName);
	}

	protected void addTimeZonesForArtifact(String artifactName, TimeZonesMap tzm){
		timeZoneMaps.put(artifactName,tzm);
	}

	protected void removeTimeZonesForArtifact(String artifactName){
		timeZoneMaps.remove(artifactName);
	}

	protected int getOffSet (String city){
		Integer tz=null;
		for (TimeZonesMap tzm : timeZoneMaps.values()){
			tz = tzm.getTimeZone(city);
		}
		if (tz!=null){			
			return tz.intValue();
		}
		else {
			return 0; 
		}
	}
}
