package org.objectweb.petals.engine.clock;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.objectweb.petals.engine.util.SourceHelper;

public class ClockListener implements Runnable {

	private DeliveryChannel channel;
	
	private MessageExchange messageExchange;
	
	private boolean running;
	
	private Logger logger;
	
	public ClockListener(DeliveryChannel channel, Logger logger) {
		super();
		this.channel = channel;
		this.logger = logger;
	}

	public void run() {
		
		running=true;
		
		mainLoop: while(running){
			try 
			{
				messageExchange = channel.accept();
				
				process(messageExchange);
	
			}
			catch (MessagingException e) {
				logger.log(Level.SEVERE,e.getMessage(),e);
				continue mainLoop;
			}
		}

	}

	private void process(MessageExchange msg)
    {
		logger.log(Level.INFO,"");
		try
		{
        	if(new QName("time").equals(msg.getOperation()))
            {
            	logger.log(Level.INFO,"The Clock service is called to return current time");
            
            	DateFormat formatter = new SimpleDateFormat("MM/dd/yyyy HH:mm:ss");
    			String en = formatter.format(new Date(System.currentTimeMillis()));
                
                
                logger.log(Level.INFO,"The Clock answer : "+en);
                
                NormalizedMessage nm = msg.createMessage();
                
                nm.setContent(SourceHelper.createSource(en));
                msg.setMessage(nm,"OUT");
                msg.setStatus(ExchangeStatus.DONE);
            }
            else
            {
            	Exception e = new Exception("only 'time' operation is recognized");
            	msg.setError(e);
            	msg.setStatus(ExchangeStatus.ERROR);
				logger.log(Level.WARNING,e.getMessage(),e);
            }
        	// Send the message (response and/or status)
			channel.send(msg);
		}
		catch (Throwable e) 
		{
        	logger.log(Level.SEVERE,e.getMessage(),e);
		} 
    }
	
	public void stopProcessing(){
		this.running = false;
	}
	
}
