package org.objectweb.petals.engine.helloworld;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;

public class Helloworld implements Component, ComponentLifeCycle {
	
	private ComponentContext context;
	
	private HelloworldListener listener;
	
	private DeliveryChannel channel;
	
	private Logger logger;
	
	private ServiceEndpoint endpointReference;
	
    private static final String SERVICE = "HelloworldService";
    private static final String ENDPOINT = "HelloworldEndpoint";
	
	public void init(ComponentContext context) throws JBIException
	{
		logger = context.getLogger("",null);
		logger.log(Level.INFO,"");
		
		this.context = context;
	}
	
	public void shutDown() throws JBIException {
		logger.log(Level.INFO,"");
		this.listener.stopProcessing();
		
	}
	
	public void start() throws JBIException {
		logger.log(Level.INFO,"");
		
		try {
			this.channel = this.context.getDeliveryChannel();
			this.listener = new HelloworldListener(this.channel,logger);
			Thread listenerThread = new Thread(this.listener);
			listenerThread.start();
			endpointReference = this.context.activateEndpoint(new QName(SERVICE), ENDPOINT); 
		} catch (MessagingException e) {
			throw new JBIException(e);
		}
		
	}
	
	public void stop() throws JBIException {
		logger.log(Level.INFO,"");
		this.listener.stopProcessing();
		
	}
	
	public ComponentLifeCycle getLifeCycle() {
		return this;
    }
	
	/**
	 * If arg0.service = HelloworldService, return the wsdl description 
	 */
	public Document getServiceDescription(ServiceEndpoint arg0) 
	{
		if(  new QName(SERVICE).equals(arg0.getServiceName()))
		{
			File wsdlFile = new File(context.getInstallRoot()+"/META-INF/HelloworldService.wsdl");
	
			return SourceHelper.createDocumentFromWSDL(wsdlFile);
		}
		return null;
	}
	
	public ServiceUnitManager getServiceUnitManager() {
		return null;
	}
	
	public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
			MessageExchange arg1) {
		
		logger.log(Level.INFO,"accept the exchange");
		return true;
	}
	
	public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
			MessageExchange arg1) {
		return false;
	}
	
	/**
	 * If requested service is HelloworldService, return the endpoint
	 */
	public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0) 
	{
		ServiceEndpoint result = null;
		
		if (arg0 != null)
		{
			Node service = SourceHelper.findChild(arg0,"service",true);

			if (service != null)
			{
				String serviceRequested = SourceHelper.getAttributeValue(service,"name");
				
				if (SERVICE.equals(serviceRequested))
				{
					result = endpointReference; 
				}
			}
		}
		return result;
	}
	
	public ObjectName getExtensionMBeanName() {
		// TODO Auto-generated method stub
		return null;
	}
	
}
