package org.objectweb.petals.engine.helloworld;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

/**
 * 
 * @author ofabre - EBM Websourcing
 *
 */
public class HelloworldListener implements Runnable {
	
	private DeliveryChannel channel;
	
	private MessageExchange messageExchange;
	
	private boolean running;
	
	private Logger logger;
	
	public HelloworldListener(DeliveryChannel channel,Logger logger){
		this.channel = channel;
		this.logger=logger;
	}
	
	public void run() 
	{
		running= true;
		
		while(running)
		{
			try 
			{
				messageExchange = channel.accept();
				
				if(messageExchange != null)
				{
					process(messageExchange);
				}
			}
			catch (MessagingException e) {
				logger.log(Level.SEVERE,e.getMessage(), e);
			}
		}
	}
	
	public void stopProcessing(){
		this.running = false;
	}
	
	
	
	private void process(MessageExchange msg)
	{
		logger.log(Level.INFO,"");
		try
		{
			// Read the IN message
			String content= null;
			
			if(new QName("printMessage").equals(msg.getOperation())
					||new QName("sayHello").equals(msg.getOperation()))
			{
				
				content = SourceHelper.createString(msg.getMessage("IN").getContent());
				
				logger.log(Level.INFO,"The Helloworld is called to say : '"+content+"'");
				
				if(StringHelper.isEmpty(content))
					throw new Exception("content is null");
				
				// If 'sayHello', the exchange requires a response, set it
				if(new QName("sayHello").equals(msg.getOperation()))
				{
					String response = "hello man ! you told me '"+content+"'";
					
					logger.log(Level.INFO,"The Helloworld answers : "+response);
					
					NormalizedMessage nm = msg.createMessage();
					
					nm.setContent(SourceHelper.createSource(response));
					msg.setMessage(nm,"OUT");
					
					msg.setStatus(ExchangeStatus.DONE);
					
					channel.send(msg);
					
				}
				else if(new QName("printMessage").equals(msg.getOperation()))
				{
					msg.setStatus(ExchangeStatus.DONE);
				
					channel.send(msg);
					
				}
				
			}
			else
			{
				Exception e = new Exception("only 'printMessage' or 'sayHello' operation is recognized");
        		msg.setError(e);
        		msg.setStatus(ExchangeStatus.ERROR);
        		logger.log(Level.WARNING,e.getMessage(),e);
			}
		}
		catch (Exception e) 
		{
			logger.log(Level.SEVERE,e.getMessage(),e);
		} 
		
	}
	
}
