/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: SourceHelper.java 94 2006-01-26 17:11:05Z rmarins $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.engine.helloworld;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.StringWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 * Sample class used to manage the mapping between 
 * <code>javax.xml.transform.Source</code> from String.
 * the mapping is a quicly "string" -> "<content>string</content>" conversion. 
 * @author Adrien LOUIS - EBM WebSourcing
  */
public class SourceHelper {

    private static final String START_TAG = "<content>";
    private static final String END_TAG = "</content>";
    
    public static Source createContentSource(String msg)
    {
        return createSource(START_TAG+msg+END_TAG);
    }
    
    public static String createStringContent(Source s) throws Exception
	{
        String content = createString(s);
        int index = content.indexOf(START_TAG);
        String result = content.substring(index+START_TAG.length());
        return result.substring(0,result.indexOf(END_TAG));
    }
    
    
	public static Source createSource(String msg)
	{
		StreamSource source = new StreamSource();
		
		byte[] msgByte = msg.getBytes();
		
		ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
		
		source.setInputStream(in);
		
		return source;
	}
	
	public static String createString(Source s) throws Exception
	{
		if(s == null || !(s instanceof StreamSource) )
		{
			throw new Exception("source is null or is not a streamsource");
		}
		
		ByteArrayInputStream bio = (ByteArrayInputStream) ((StreamSource)s).getInputStream();
		
		if(bio.available() == 0)
		{
			throw new Exception("source is empty");
		}
		
		byte[] buff = new byte[bio.available()];
		
		bio.read(buff);
		
		String str = new String(buff);
		
		return str;
	}
	
	public static Document createDocumentFromWSDL(File wsdlFile)
	{
		Document doc = null;
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			doc = builder.parse(wsdlFile);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return doc;
	}
	
	/**
	 * Search for a child with the given nodeName.
	 * If recursive, search in all the child of firdt level,
	 * then if not found, search in the 2nd level of the first child, ...
	 * @param parent
	 * @param nodeName
	 * @param recursive
	 * @return
	 */
	public static Node findChild(Node parent, String nodeName, boolean recursive)
	{
		Node result = null;
		
		if(parent != null && nodeName != null)
		{
			NodeList nl =  parent.getChildNodes();
			
			for (int i =0; i<nl.getLength() && result == null;i++)
			{
				if (nodeName.equals(nl.item(i).getNodeName()))
				{
					result = nl.item(i);
				}		
			}
			
			// now, search recursively if required
			if(result == null && recursive)
			{
				for (int i =0; i<nl.getLength() && result == null;i++)
				{
					result = findChild(nl.item(i), nodeName, true);
				}	
			}
		}
		return result;
	}
	
	/**
     * Return the value of the attribute in the node
     * @param n the node
     * @param attName the name of the attribute
     * @return the value of the attribute, null if not found
     */
	public static String getAttributeValue(Node n, String attName)
    {
        NamedNodeMap atts = n.getAttributes();
        
        Node att = atts.getNamedItem(attName);
        
        if(att != null)
        {
            return att.getNodeValue();
        }
        
        return null;
    }
    
    /**
     * Return the <i>text</i> element of a node
     * @param node
     * @return
     */
    public static String getTextContent(Node node)
    {
         NodeList list = node.getChildNodes();
         
         for(int i= 0; i<list.getLength();i++)
         {
             if(list.item(i).getNodeType() == Node.TEXT_NODE )
             {
                 return list.item(i).getNodeValue();
             }
         }
         return null;
    }
    
    /**
     * Return the next sibling of a node,
     * regardless <i>text</i> node
     * @param node
     * @return
     */
    public static Node getNextSibling(Node node)
    {
        Node result = node.getNextSibling();
        
        while(result.getNodeType() == Node.TEXT_NODE)
        {
            result = result.getNextSibling();
        }
        return result;
    }
    

    /**
     * Return the first child of a node,
     * regardless <i>text</i> node
     * @param node
     * @return
     */
    public static Node getFirstChild(Node node)
    {
        Node result = node.getFirstChild();
        
        while(result.getNodeType() == Node.TEXT_NODE)
        {
            result = result.getNextSibling();
        }
        return result;
    }
    
    public static String parseToString(Node node) throws TransformerException {
		Transformer transformer = TransformerFactory.newInstance().newTransformer();
		StringWriter stringWriter = new StringWriter(128);
		transformer.transform(new DOMSource(node), new StreamResult(stringWriter));
		StringBuffer buffer =  stringWriter.getBuffer();
		return buffer.toString();
	}
}
