package org.objectweb.petals.engine.sampleclient;

import java.io.ByteArrayInputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOptionalOut;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Source;

import org.objectweb.petals.engine.util.SourceHelper;
import org.objectweb.petals.engine.sampleclient.gui.Console;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.xml.sax.InputSource;

public class SampleClient implements Component, ComponentLifeCycle {
	
	private ComponentContext context;
	
	private DeliveryChannel channel;
	
	@SuppressWarnings("unused")
	private ServiceEndpoint endpointReference;
	
	private SampleClientListener listener;
	
	private Console console = new Console(this);
	
	private Logger logger;
	
    private static final String SERVICE = "sampleclientComponentService";
	
	
	public void init(ComponentContext context) throws JBIException {
		
		this.context = context;
		logger = context.getLogger("",null);
		logger.log(Level.INFO,"");
	}

	public void shutDown() throws JBIException {
		logger.log(Level.INFO,"");
		this.listener.stopProcessing();
		console.setVisible(false);
		console.dispose();

	}

	public void start() throws JBIException {
		logger.log(Level.INFO,"");
		
		try {
			this.channel = this.context.getDeliveryChannel();
			this.listener = new SampleClientListener(this.channel,console,logger); 
			Thread listenerThread = new Thread(this.listener);
			listenerThread.start();
			endpointReference = this.context.activateEndpoint(new QName(SERVICE), context.getComponentName()+"Endpoint");
			
			console.setVisible(true);
		} catch (MessagingException e) {
			throw new JBIException(e);
		}

	}

	public void stop() throws JBIException {
		logger.log(Level.INFO,"");
		this.listener.stopProcessing();

	}
	
	
	public ComponentLifeCycle getLifeCycle() {
		return this;
	}

	public Document getServiceDescription(ServiceEndpoint arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	public ServiceUnitManager getServiceUnitManager() {
		return null;
	}

	public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
			MessageExchange arg1) {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
			MessageExchange arg1) {
		logger.log(Level.INFO,"accept the exchange");
		return true;
	}

	public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	public ObjectName getExtensionMBeanName() {
		// TODO Auto-generated method stub
		return null;
	}

	
	
	public static final String INONLY = "InOnly";
	public static final String INOUT = "InOut";
	public static final String INOPTIONALOUT = "InOptionalOut";
	public static final String ROBUSTINONLY = "RobustInOnly";
				
						
	public void send(String service,String op,String content,String type,long syncTime)
	{
		logger.log(Level.INFO,"");
		try 
		{
            Source source = SourceHelper.createContentSource(content);
            
            MessageExchange msg = null;
            
            if(type.equals(INONLY)) msg = context.getDeliveryChannel().createExchangeFactory().createInOnlyExchange();
            else if(type.equals(INOUT)) msg = context.getDeliveryChannel().createExchangeFactory().createInOutExchange();
            else if(type.equals(INOPTIONALOUT)) msg = context.getDeliveryChannel().createExchangeFactory().createInOptionalOutExchange();
            else if(type.equals(ROBUSTINONLY)) msg = context.getDeliveryChannel().createExchangeFactory().createRobustInOnlyExchange();


            NormalizedMessage nm = msg.createMessage();
            
            nm.setContent(source);
            
            msg.setService(new QName(service));
            
            msg.setOperation(new QName(op));
            
            if(type.equals(INONLY)) ((InOnly)msg).setInMessage(nm); 
            else if(type.equals(INOUT)) ((InOut)msg).setInMessage(nm);
            else if(type.equals(INOPTIONALOUT)) ((InOptionalOut)msg).setInMessage(nm);
            else if(type.equals(ROBUSTINONLY)) ((RobustInOnly)msg).setInMessage(nm);
            
            if(syncTime<0)
            {
            	context.getDeliveryChannel().send(msg);
            }
            else
            {
            	context.getDeliveryChannel().sendSync(msg,syncTime);
            	listener.process(msg);
            }
        } 
		catch (Exception e) 
        {
			logger.log(Level.SEVERE,e.getMessage());
			console.showError(e);
        }
	}

	public ServiceEndpoint[] getEndpoints()
	{
		return context.getEndpoints(null);
		
	}
	
	public ServiceEndpoint resolveDescription(String text)
	{
		InputSource source = createSource(text);
		
		DocumentFragment result = null;

		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			
			DocumentBuilder builder = factory.newDocumentBuilder();
			
			Document document = builder.parse(source);

			result = document.createDocumentFragment();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return context.resolveEndpointReference(result);
		
	}
	
	public String getDescription(ServiceEndpoint se)
	{
		Document doc;
		try {
			doc = context.getEndpointDescriptor(se);
			return SourceHelper.parseToString(doc);
		} catch (Throwable e) {
			console.showError(e);
		}
		return null;
	}
	
	public String getAsReference(ServiceEndpoint se)
	{
		try {
			DocumentFragment doc = se.getAsReference(null);
			return SourceHelper.parseToString(doc);
		} catch (Throwable e) {
			console.showError(e);
		}
		return null;
	}
	
	
	
	protected InputSource createSource(String msg)
	{
		InputSource source = new InputSource();
		
		byte[] msgByte = msg.getBytes();
		
		ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
		
		source.setByteStream(in);
		
		return source;
	}
}
