/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ZipUtil.java 98 2006-02-24 16:18:48Z gblondelle $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.util;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URI;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import org.objectweb.petals.PetalsException;

/**
 * Contains utilities methods in relation to Zip operations.
 *
 * @version	$Rev: 98 $ $Date: 2006-02-24 17:18:48 +0100 (ven., 24 févr. 2006) $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class ZipUtil {
	
	/**
	 * Do not construct.
	 */
	private ZipUtil() {
		super();
	}
	
	/**
	 * See you must delete the temp file after using.
	 * 
	 * @param zipFile
	 * @param entryName
	 * @return a temp File read from the Zip entry, or <code>null</code> if
	 * no entry was found.
	 */
	public static File getEntryAsTemp(ZipFile zipFile, String entryName) {
		
		if (zipFile == null) {
			throw new IllegalArgumentException("Invalid null ZipFile.");
			
		} else if (entryName == null || "".equals(entryName)) {
			throw new IllegalArgumentException(
			"Invalid Zip entry was specified: null or empty.");
		}
		
		File tempFile = null;
		
		InputStream zipStream = null;
		try {
			ZipEntry theEntry = zipFile.getEntry(entryName);
			
			if (theEntry == null) {
				return null;
			}
			
			zipStream = zipFile.getInputStream(theEntry);
			
			tempFile = File.createTempFile("petals-", "-zip.tmp");
			
			NioUtil.copyStreamToFile(zipStream, tempFile);
			
		} catch (IOException e) {
			// do nothing else, but return null
			e.printStackTrace();
			
		} finally {
			try {
				if (zipStream != null) {
					zipStream.close();
				}
			} catch (IOException ioe) {
				// do nothing
			}
		}
		
		return tempFile;
	}
	
	public static File unpackEntry(ZipFile zipFile, String entry,
			String location) {
		return null;
	}
	
	/**
	 * 
	 * @param zipFile
	 * @param aDirectory
	 * @throws PetalsException error during exploding the Zip archive into the
	 * directory. 
	 */
	public static void explodeIntoDirectory(ZipFile zipFile, File aDirectory)
	throws PetalsException {
		
		if (zipFile == null || aDirectory == null) {
			throw new IllegalArgumentException(
			"The zip file or directory must not be null.");
		} else if (!aDirectory.exists() || !aDirectory.isDirectory()) {
			throw new IllegalArgumentException(
			"The given file is not a directory or doesn't exist.");
		}
		
		Enumeration entries = zipFile.entries();
		
		while (entries.hasMoreElements()) {
			ZipEntry anEntry = (ZipEntry) entries.nextElement();
			
			File unpackFile = new File(aDirectory, anEntry.getName());
			
			File parentDir = unpackFile.getParentFile();
			if (!parentDir.exists()) {
				
				if (!parentDir.mkdirs()) {
					throw new PetalsException(
							"Failed to explode zip into directory: " +
							anEntry.getName());
				}
			}
			
			if (!anEntry.isDirectory()){
				InputStream zipStream = null;
				try {
					zipStream = zipFile.getInputStream(anEntry);
					NioUtil.copyStreamToFile(zipStream, unpackFile);
					
				}
				catch (IOException ioe) {
					throw new PetalsException(
							"Failed to unpack a Zip file entry: " +
							anEntry.getName(), ioe);
					
				}
				finally {
					if (zipStream != null) {
						try {
							zipStream.close();
						} catch (IOException e) {
							// do nothing
						}
					}
				}
			}
			
		}
		
	}
	
	public static ZipFile openZipFile(URI archiveURI) throws PetalsException {
		
		ZipFile zipArchive = null;
		String msg;
		
		try {
			
			File archiveFile = new File(archiveURI);
			zipArchive = new ZipFile(archiveFile);
			
		}
		catch (ZipException ze) {
			msg = "Error opening Zip file. Location: " + archiveURI.toString();
			
			throw new PetalsException(msg, ze);
			
		}
		catch (IOException ioe) {
			msg = "Unexpected I/O exception handling Zip file. Location : "
				+ archiveURI.toString();
			throw new PetalsException(msg, ioe);
		}
		
		return zipArchive;
	}
	
	public static boolean hasEntry(ZipFile archive, String entryName) {
		ZipEntry theEntry = archive.getEntry(entryName);
		return (theEntry != null);
	}
	
}
