/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ComponentContextImpl.java 216 2006-04-13 14:18:14Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.component;

import java.util.Iterator;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.management.MBeanNames;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.MBeanServer;
import javax.naming.InitialContext;
import javax.xml.namespace.QName;

import org.objectweb.petals.jbi.descriptor.JBIDescriptor;
import org.objectweb.petals.jbi.management.LifeCycle;
import org.objectweb.petals.jbi.management.ManagerService;
import org.objectweb.petals.jbi.messaging.DeliveryChannelImpl;
import org.objectweb.petals.jbi.messaging.MessagingService;
import org.objectweb.petals.jbi.registry.InternalEndpoint;
import org.objectweb.petals.jbi.registry.RegistryManager;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;


/**
 * @version	$Revision: 216 $ $Date: 2006-04-13 16:18:14 +0200 (jeu., 13 avr. 2006) $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class ComponentContextImpl implements javax.jbi.component.ComponentContext {
	
	private JBIDescriptor descriptor;
	
	private String installationRoot;
	
	private String workspaceRoot;
	
	private RegistryManager registryManager;
	
	private DeliveryChannelImpl deliveryChannel; 
	
	private Component jbiComponent;
	
	private ManagerService manager;
	
	public ComponentContextImpl() {
	}
	
	/**
	 * 
	 * @param descriptor
	 */
	public ComponentContextImpl(
			JBIDescriptor descriptor, 
			String installationRoot,
			String workspaceRoot, 
			MessagingService messagingService,
			ManagerService manager) {
		super();
		this.descriptor = descriptor;
		this.installationRoot = installationRoot;
		this.registryManager = messagingService.getRegistry();
		this.deliveryChannel = (DeliveryChannelImpl) messagingService.loadDeliveryChannel(this);
		this.manager = manager;
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#activateEndpoint(javax.xml.namespace.QName, java.lang.String)
	 */
	public ServiceEndpoint activateEndpoint(QName serviceName,
			String endpointName) throws JBIException {
		
		return registryManager.registerInternalEndpoint(serviceName,
				endpointName,this);
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#deactivateEndpoint(javax.jbi.servicedesc.ServiceEndpoint)
	 */
	public void deactivateEndpoint(ServiceEndpoint endpoint)
	throws JBIException {
		registryManager.deregisterInternalEndpoint(endpoint);
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#deregisterExternalEndpoint(javax.jbi.servicedesc.ServiceEndpoint)
	 */
	public void deregisterExternalEndpoint(ServiceEndpoint externalEndpoint)
	throws JBIException {
		registryManager.deregisterExternalEndpoint(externalEndpoint);
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#getComponentName()
	 */
	public String getComponentName() {
		return descriptor.getComponent().getIdentification().getName();
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#getDeliveryChannel()
	 */
	public DeliveryChannel getDeliveryChannel() throws MessagingException {
		return deliveryChannel;
	}
	
	public void setDeliveryChannel(DeliveryChannelImpl deliveryChannel) {
		this.deliveryChannel = deliveryChannel;
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#getEndpoint(javax.xml.namespace.QName, java.lang.String)
	 */
	public ServiceEndpoint getEndpoint(QName service, String name) 
	{
		return registryManager.getInternalEndpoint(service, name);
	}
	
	/**
	 * TODO getEndpointDescriptor works only with implementation InternalEndpoint
	 * @see javax.jbi.component.ComponentContext#getEndpointDescriptor(javax.jbi.servicedesc.ServiceEndpoint)
	 */
	public Document getEndpointDescriptor(ServiceEndpoint endpoint)
	throws JBIException 
	{
		InternalEndpoint ep = (InternalEndpoint)registryManager.getInternalEndpoint(endpoint.getServiceName(), endpoint.getEndpointName());
		
		Document result = null;
		
		if(ep != null)
		{
			Component comp = ep.getComponentContext().getComponent();
			
			result = comp.getServiceDescription(ep);
		}
		else
		{
			throw new JBIException("Invalid endpoint reference.");
		}
		return result;
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#getEndpoints(javax.xml.namespace.QName)
	 */
	public ServiceEndpoint[] getEndpoints(QName interfaceName) {
		
		return registryManager.getInternalEndpointsForInterface(interfaceName);
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#getEndpointsForService(javax.xml.namespace.QName)
	 */
	public ServiceEndpoint[] getEndpointsForService(QName serviceName) {
		
		return registryManager.getInternalEndpointsForService(serviceName);
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#getExternalEndpoints(javax.xml.namespace.QName)
	 */
	public ServiceEndpoint[] getExternalEndpoints(QName interfaceName) {
		
		return registryManager.getExternalEndpointsForInterface(interfaceName);
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#getExternalEndpointsForService(javax.xml.namespace.QName)
	 */
	public ServiceEndpoint[] getExternalEndpointsForService(QName serviceName) {
		
		return registryManager.getExternalEndpointsForService(serviceName);
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#getInstallRoot()
	 */
	public String getInstallRoot() {
		return installationRoot;
	}
	
	/**
	 * TODO return a new java logger, not manage by the container
	 * @see javax.jbi.component.ComponentContext#getLogger(java.lang.String, java.lang.String)
	 */
	public Logger getLogger(String suffix, String resourceBundleName)
	throws MissingResourceException, JBIException 
	{
		Logger l = Logger.getLogger(descriptor.getComponent().getIdentification().getName(),resourceBundleName);
		if (l == null)
			l = Logger.getAnonymousLogger();
		if (l == null)
			throw new JBIException("No logger can be created");
		return l;
		
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#getMBeanNames()
	 */
	public MBeanNames getMBeanNames() {
		
		return manager.getMBeanNames();
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#getMBeanServer()
	 */
	public MBeanServer getMBeanServer() {
		
		return manager.getMBeanServer();
	}
	
	/**
	 * TODO not implemented
	 * @see javax.jbi.component.ComponentContext#getNamingContext()
	 */
	public InitialContext getNamingContext() {
		throw new NoSuchMethodError();
	}
	
	/**
	 * TODO not implemented
	 * @see javax.jbi.component.ComponentContext#getTransactionManager()
	 */
	public Object getTransactionManager() {
		throw new NoSuchMethodError();
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#getWorkspaceRoot()
	 */
	public String getWorkspaceRoot() 
	{
		return workspaceRoot;
	}
	
	/**
	 * @see javax.jbi.component.ComponentContext#registerExternalEndpoint(javax.jbi.servicedesc.ServiceEndpoint)
	 */
	public void registerExternalEndpoint(ServiceEndpoint externalEndpoint)
	throws JBIException {
		
		registryManager.registerExternalEndpoint(externalEndpoint);
	}
	
	/**
	 * TODO resolveEndpointReference : refactor
	 * @see javax.jbi.component.ComponentContext#resolveEndpointReference(org.w3c.dom.DocumentFragment)
	 */
	public ServiceEndpoint resolveEndpointReference(DocumentFragment epr) 
	{
		ServiceEndpoint result = null;
		
		// Search for all engines components if one can resolve the EPR.
		Map engines = manager.getSync_engines();
		
		synchronized(engines)
		{
			for (Iterator iter = engines.values().iterator(); iter.hasNext() && result == null;)
			{
				LifeCycle lifeCycle = (LifeCycle) iter.next();
				
				result = lifeCycle.getComponent().resolveEndpointReference(epr);
			}
		}
		
		if(result == null)
		{
			// Search for all binding components if one can resolve the EPR.
			Map bindings = manager.getSync_bindings();
			
			synchronized(bindings)
			{
				for (Iterator iter = bindings.values().iterator(); iter.hasNext() && result == null;)
				{
					LifeCycle lifeCycle = (LifeCycle) iter.next();
					
					result = lifeCycle.getComponent().resolveEndpointReference(epr);
				}
			}
		}
		return result;
		
	}
	
	public Component getComponent() {
		return jbiComponent;
	}
	
	public void setComponent(Component jbiComponent) {
		this.jbiComponent = jbiComponent;
	}
	
	
}
