package org.objectweb.petals.jbi.descriptor;

import javax.xml.namespace.QName;

/**
 * The provides elements is used to declare a service provided by a service
 * unit. A provided service is declared with the following attributes:
 * <ul>
 * <li><b>interface-name.</b> This qualified name indicates the type of
 * service to be provided.</li>
 * <li><b>service-name and endpoint-name.</b>This pair of attributes
 * declares the name of the endpoint that the service unit will, after
 * deployment, activate with JBI.</li>
 * </ul>
 *
 * @version	$Rev: 76 $ $Date: 2006-01-17 18:15:44 +0100 (mar., 17 janv. 2006) $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class Provides {

	/**
	 * Qualified interface name.
	 */
	private QName interfaceName;

	/**
	 * Service name to be provided.
	 */
	private QName serviceName;

	/**
	 * Endpoint name to be provided.
	 */
	private String endpointName;

	/**
	 * Default constructor.
	 */
	Provides() {
		super();
	}

	//
	// Getters and Setters
	//

	/**
	 * Returns the endpoint name that the service unit will, after
	 * deployment, activate with JBI.
	 *
	 * @return <code>String</code> the endpoint name.
	 */
	public String getEndpointName() {
		return endpointName;
	}

	/**
	 * Return the qualified name indicating the type of service to be
	 * provided.
	 *
	 * @return <code>QName</code> the interface name.
	 */
	public QName getInterfaceName() {
		return interfaceName;
	}

	/**
	 * Returns the service name that the service unit will, after
	 * deployment, activate with JBI.
	 *
	 * @return <code>QName</code> the service name.
	 */
	public QName getServiceName() {
		return serviceName;
	}

	/**
	 * Set the endpoint name which will be activated in JBI.
	 *
	 * @param endpointName the endpoint name.
	 */
	void setEndpointName(String endpointName) {
		this.endpointName = endpointName;
	}

	/**
	 * Set the interface name indicating the type of service to be provided.
	 *
	 * @param interfaceName the interface name.
	 */
	void setInterfaceName(QName interfaceName) {
		this.interfaceName = interfaceName;
	}

	/**
	 * Set the service name which will be activated in JBI.
	 *
	 * @param serviceName the service name.
	 */
	void setServiceName(QName serviceName) {
		this.serviceName = serviceName;
	}
}