package org.objectweb.petals.jbi.descriptor;

import java.util.List;

/**
 * The Service Assembly production is used to describe a service assembly
 * deployment package. This deployment descriptor describes the contained
 * service units, and to which components they are to be deployed.
 *
 * @version	$Rev: 76 $ $Date: 2006-01-17 18:15:44 +0100 (mar., 17 janv. 2006) $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class ServiceAssembly {

	/**
	 * Shared Library identification data.
	 */
	private Identification identification;

	/**
	 * Individual deployments to be made to components.
	 */
	private List<ServiceUnit> serviceUnits;

	/**
	 * Contains declarations of service interconnection metadata, used to
	 * map static service consumers to service providers.
	 */
	private List<Connection> connections;

	/**
	 * Default constructor.
	 */
	ServiceAssembly() {
		super();
	}

	//
	// Getters and Setters
	//
	
	/**
	 * Returns service assembly's identification data.
	 *
	 * @return <code>Identification</code> the SA's identification.
	 */
	public Identification getIdentification() {
		return identification;
	}

	/**
	 * Set the service assembly identification data.
	 *
	 * @param identification The identification to set.
	 */
	void setIdentification(Identification identification) {
		this.identification = identification;
	}

	/**
	 * Returns service units elements describing the individual deployments
	 * to be made to components.
	 *
	 * @return <code>List<ServiceUnit></code> the individual deployments.
	 */
	public List<ServiceUnit> getServiceUnits() {
		return serviceUnits;
	}

	/**
	 * Set Service Assembly individual deployments.
	 *
	 * @param serviceUnits <code>List<ServiceUnit></code> the individual
	 * deployments.
	 */
	void setServiceUnits(List<ServiceUnit> serviceUnits) {
		this.serviceUnits = serviceUnits;
	}

	/**
	 * Returns optional elements containing declarations of service
	 * interconnection metadata, used to map static service consumers to
	 * service providers.
	 *
	 * @return <code>List<Connection></code> the interconnections between
	 * service consumers and providers mapping.
	 */
	public List<Connection> getConnections() {
		return connections;
	}

	/**
	 * Set interconnections between service consumers and providers mapping.
	 *
	 * @param connections <code>List<Connection></code> the interconnection
	 * mapping list.
	 */
	void setConnections(List<Connection> connections) {
		this.connections = connections;
	}

}