package org.objectweb.petals.jbi.descriptor;


/**
 * The Service Unit production is used to describe a service component
 * within a service assembly deployment package.
 *
 * @version	$Rev: 76 $ $Date: 2006-01-17 18:15:44 +0100 (mar., 17 janv. 2006) $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class ServiceUnit {

	/**
	 * Service Unit identification data.
	 */
	private Identification identification;

	/**
	 * Name of the artifact archive file.
	 */
	private String targetArtifactsZip;

	/**
	 * Component name.
	 */
	private String targetComponentName;

	/**
	 * Default constructor.
	 */
	ServiceUnit() {
		super();
	}

	//
	// Getters and Setters
	//
	
	/**
	 * Returns service unit's identification data.
	 *
	 * @return <code>Identification</code> the SU's identification.
	 */
	public Identification getIdentification() {
		return identification;
	}

	/**
	 * Set the service unit identification data.
	 *
	 * @param identification The identification to set.
	 */
	void setIdentification(Identification identification) {
		this.identification = identification;
	}

	/**
	 * Returns the name of the artifact archive file within the deployment
	 * package that will be deployed.
	 *
	 * @return <code>String</code> the artifact archive file.
	 */
	public String getTargetArtifactsZip() {
		return targetArtifactsZip;
	}

	/**
	 * Returns the name of the component that will have the
	 * <i>artifacts zip</i> deployed to it.
	 *
	 * @return <code>String</code> the component name.
	 */
	public String getTargetComponentName() {
		return targetComponentName;
	}

	/**
	 * Set the target artifacts archive file within the deployment package.
	 *
	 * @param targetArtifactsZip <code>String</code> the archive file.
	 */
	void setTargetArtifactsZip(String targetArtifactsZip) {
		this.targetArtifactsZip = targetArtifactsZip;
	}

	/**
	 * Set the target component name to which the artifacts archive file
	 * is to be deployed.
	 *
	 * @param targetComponentName <code>String</code> the component name.
	 */
	void setTargetComponentName(String targetComponentName) {
		this.targetComponentName = targetComponentName;
	}

}