package org.objectweb.petals.jbi.descriptor;

import java.util.List;

/**
 * Shared Library production is used to describe a shared library
 * installation package.
 *
 * @version	$Rev: 76 $ $Date: 2006-01-17 18:15:44 +0100 (mar., 17 janv. 2006) $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class SharedLibrary {

	/**
	 * Shared Library identification data.
	 */
	private Identification identification;

	/**
	 * Shared Library class loading delegation model:
	 * <code>"self-first"</code> or <code>"parent-first"</code>.
	 */
	private String classLoaderDelegation;

	/**
	 * Optional shared library version.
	 */
	private String version;

	/**
	 * Shared Library Class Path elements.
	 */
	private List<String> sharedLibraryClassPath;

	/**
	 * Default constructor.
	 */
	SharedLibrary() {
		super();
	}

	//
	// Getters and Setters
	//

	/**
	 * Returns component's identification data.
	 *
	 * @return <code>Identification</code> the component's identification.
	 */
	public Identification getIdentification() {
		return identification;
	}

	/**
	 * Set the component identification data.
	 *
	 * @param identification The identification to set.
	 */
	void setIdentification(Identification identification) {
		this.identification = identification;
	}

	/**
	 * Returns the class loading delegation model of this Shared Library.
	 * If <code>null</code> the "parent-first" default delegation model
	 * must be used.
	 *
	 * @return <code>String</code> the delegation model:
	 * <code>"parent-first"</code> or <code>"self-first"</code>.
	 */
	public String getClassLoaderDelegation() {
		return classLoaderDelegation;
	}

	/**
	 * Set the Shared Library class loading delegation model.
	 *
	 * @param delegationModel the delegation model to set:
	 * <code>"parent-first"</code> or <code>"self-first"</code>.
	 */
	void setClassLoaderDelegation(String delegationModel) {
		this.classLoaderDelegation = delegationModel;
	}

	/**
	 * @return Returns the sharedLibraryClassPath.
	 */
	public List<String> getSharedLibraryClassPath() {
		return sharedLibraryClassPath;
	}

	/**
	 * @return Returns the version.
	 */
	public String getVersion() {
		return version;
	}

	/**
	 * @param sharedLibraryClassPath The sharedLibraryClassPath to set.
	 */
	void setSharedLibraryClassPath(List<String> sharedLibraryClassPath) {
		this.sharedLibraryClassPath = sharedLibraryClassPath;
	}

	/**
	 * @param version The version to set.
	 */
	void setVersion(String version) {
		this.version = version;
	}
}