/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: AdminService.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.management;

import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_JMXADMIN_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.LOGGER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.MONOLOG_FACTORY_NAME;

import java.util.Iterator;
import java.util.Map;

import javax.management.ObjectName;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;



/**
 * TODO AdminService : check the exceptions catch
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class AdminService implements AdminServiceMBean, BindingController, LifeCycleController
{
	
    private static final String[] CLIENT_INTERFACES = new String[] { JBI_JMXADMIN_NAME };
    
	/**
	 * Component lifecycle state.
	 */
	private String fcState;
    
	/**
	 * JBI :: JMX Agent Admin
	 */
    private ManagerService manager;
    
    /**
     * Monolog
     */
    private Logger logger;
    
    /**
	 * Logger wrapper
	 */
	private LoggingUtil log;
    
    /**
     * Default constructor
     */
    public AdminService()
    {
    	super();
    }
    
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#getBindingComponents()
     */
    public ObjectName[] getBindingComponents()
    {
        log.start();
        
        ObjectName[] names = null;
        
        Map<ObjectName,ComponentLifeCycle> bindings = manager.getSync_bindings();
        
        names = (ObjectName[])bindings.keySet().toArray(new ObjectName[0]);
        
        return names;
    }
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#getComponentByName(java.lang.String)
     */
    public ObjectName getComponentByName(String name)
    {
        log.call();
        
        ObjectName on = null;
        
        // search in bindings
        
        on = getBindingComponentByName(name);
        
        if(on == null)
        {
        	// search in engines
        
        	on = getEngineComponentByName(name);
        }        	
        return on;
    }
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#getEngineComponents()
     */
    @SuppressWarnings("unchecked")
	public ObjectName[] getEngineComponents()
    {
        log.call();
        
        ObjectName[] names = null;
        
        Map engines = manager.getSync_engines();
        
        names = (ObjectName[])engines.keySet().toArray(new ObjectName[0]);
        
        return names;
    }
    
    /**
     * TODO getSystemInfo
     * @see javax.jbi.management.AdminServiceMBean#getSystemInfo()
     */
    public String getSystemInfo()
    {
        return "Petals JBI Container - version: " + 
        Package.getPackage("org.objectweb.petals").getImplementationVersion();
    }
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#getSystemService(java.lang.String)
     */
    public ObjectName getSystemService(String serviceName)
    {
    	log.call();
    	
    	ObjectName result = null;
    	
    	Map systemServices = manager.getSync_systemServices();
        
    	synchronized(systemServices)
        {
            for (Iterator iter = systemServices.values().iterator(); iter.hasNext() && result == null;)
            {
                LifeCycle lifeCycle = (LifeCycle) iter.next();
                
                if (lifeCycle.getName().equals(serviceName))
                {
                	result = lifeCycle.getMBeanName();
                }
            }
        }
    	return result;
    }
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#getSystemServices()
     */
    @SuppressWarnings("unchecked")
	public ObjectName[] getSystemServices()
    {
        log.call();
        
        ObjectName[] names = null;
        
        Map services = manager.getSync_systemServices();
                
        names = (ObjectName[])services.keySet().toArray(new ObjectName[0]);
        
        return names;
    }
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#isBinding(java.lang.String)
     */
    public boolean isBinding(String componentName)
    {
        return manager.getSync_bindings().containsKey(componentName);
    }
    
    /**
     * @see javax.jbi.management.AdminServiceMBean#isEngine(java.lang.String)
     */
    public boolean isEngine(String componentName)
    {
        return manager.getSync_engines().containsKey(componentName);
    }
    
    /**
     * Return the engine component associated with this name.
     * @param name
     * @return The related objectName, or null
     */
    private ObjectName getEngineComponentByName(String name)
    {
    	log.call();
    	
    	ObjectName result = null;
    	
    	Map engines = manager.getSync_engines();
        
    	synchronized(engines)
        {
            for (Iterator iter = engines.values().iterator(); iter.hasNext() && result == null;)
            {
                LifeCycle lifeCycle = (LifeCycle) iter.next();
                
                if (lifeCycle.getName().equals(name))
                {
                	result = lifeCycle.getMBeanName();
                }
            }
        }
    	return result;
    }
    
    /**
     * Return the binding component associated with this name.
     * @param name
     * @return The related objectName, or null
     */
    private ObjectName getBindingComponentByName(String name)
    {
    	log.call();
    	
    	ObjectName result = null;
    	
    	Map bindings = manager.getSync_bindings();
        
    	synchronized(bindings)
        {
            for (Iterator iter = bindings.values().iterator(); iter.hasNext() && result == null;)
            {
                LifeCycle lifeCycle = (LifeCycle) iter.next();
                
                if (lifeCycle.getName().equals(name))
                {
                	result = lifeCycle.getMBeanName();
                }
            }
        }
    	return result;
    }
    
    
    
    
    
    
    
    
    
    
	// ----------------------------------------------------------------------
	//   Fractal BindingController implementation
	// ----------------------------------------------------------------------

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#listFc()
	 */
	public String[] listFc() {
		return CLIENT_INTERFACES;
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
	 */
	public Object lookupFc(String clientItfName)
			throws NoSuchInterfaceException {

		if (JBI_JMXADMIN_NAME.equals(clientItfName)) {
			return manager;
		}
		else {
			throw new NoSuchInterfaceException(clientItfName);
		}
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
	 */
	public void bindFc(String clientItfName, Object serverItf)
			throws NoSuchInterfaceException, IllegalBindingException,
			IllegalLifeCycleException {
		
		if (LOGGER_NAME.equals(clientItfName)) 
		{
		     logger = (Logger) serverItf;
		} 
		else if (MONOLOG_FACTORY_NAME.equals(clientItfName))
		{
			//nothing
		}
		else if (JBI_JMXADMIN_NAME.equals(clientItfName)) 
		{
			manager = (ManagerService) serverItf;
		}
		else 
		{
			throw new NoSuchInterfaceException(clientItfName);
		}
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
	 */
	public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
			IllegalBindingException, IllegalLifeCycleException {

		if (LOGGER_NAME.equals(clientItfName)){
		     logger = null;
		} else if (MONOLOG_FACTORY_NAME.equals(clientItfName)){
			//nothing
		}else if (JBI_JMXADMIN_NAME.equals(clientItfName)) {
			manager = null;
		}
		else {
			throw new NoSuchInterfaceException(clientItfName);
		}
	}


	// ----------------------------------------------------------------------
	//   Fractal LifecycleController implementation
	// ----------------------------------------------------------------------

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
	 */
	public String getFcState() {
		return (STARTED.equals(fcState) ? STARTED : STOPPED);
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
	 */
	public void startFc() throws IllegalLifeCycleException {

		log = new LoggingUtil(logger);
		
		fcState = STARTED;
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
	 */
	public void stopFc() throws IllegalLifeCycleException {

		fcState = STOPPED;
	}
    
}