package org.objectweb.petals.jbi.management;

import java.util.Map;

import javax.management.MBeanServer;
import javax.management.ObjectName;



/**
 * Manager is the entry point for register and unregister elements in the JMX environment
 * (elements are JBI binding/engine components and systemServices
 * components).
 * Related MBeans are created when registering and unregistering elements.
 * 
 * <code>Manager</code> owns the JMX management services offered by the JBI container 
 * to administrative clients. 
 * Management services are :
 * - AdminService (referenced in the JMX server as "admin");
 * - InstallationService (referenced in the JMX server as "installation");
 * - DeploymentService (referenced in the JMX server as "deployment");
 * 
 * <code>Manager</code> is the central point for accessing the mBeans associated 
 * to the JBI container elements.
 * 
 * <code>ManagementException</code> exceptions are thrown when
 * problems for accessing the JMX server occur.
 * 
 * @author ofabre
 *
 */
public interface ManagerService {
	
	MBeanNamesImpl getMBeanNames();
	MBeanServer getMBeanServer();
	Map<ObjectName,ComponentLifeCycle> getSync_bindings();
	Map<ObjectName,ComponentLifeCycle> getSync_engines();
	Map<ObjectName,LifeCycle> getSync_systemServices();
	ObjectName registerBindingComponent(ComponentLifeCycle lifeCycle) throws ManagementException;
	void unregisterBindingComponent(ObjectName name) throws ManagementException;
	ObjectName registerEngineComponent(ComponentLifeCycle lifeCycle) throws ManagementException;
	void unregisterEngineComponent(ObjectName name) throws ManagementException;
	
	/**
     * Find the {@link LifeCycle} of a JBI Installable Component
     * by its unique name.
     * 
     * @param name
     *            the name of the engine or binding component; must be non-null
     *            and non-empty
     * @return the {@link LifeCycle} of the component, or
     *         <code>null</code> if there is no such component with the given
     *         <code>name</code>
	 */
	LifeCycle getComponentByName(String name);
	void registerServiceAssembly(String saName, ServiceAssemblyLifeCycle serviceAssemblyLifeCycle);
	void unregisterServiceAssembly(String saName) throws ManagementException;
	ServiceAssemblyLifeCycle getServiceAssemblyByName(String saName);
	Map<String, ServiceAssemblyLifeCycle> getSync_serviceAssemblyLifeCycles();
	ObjectName registerInstaller(Installer installerMBean) throws ManagementException;
	void unregisterInstaller(ObjectName name) throws ManagementException;
	Map<ObjectName, Installer> getSync_installers();
	ObjectName getInstallerByName(String name);
}
