/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: MessageExchangeImpl.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.messaging;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Set;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

//import org.objectweb.petals.routing.Itinerary;


/**
 * @author Adrien LOUIS - EBM WebSourcing
 * @author Gael BLONDELLE - EBM WebSourcing
 */
public class MessageExchangeImpl implements MessageExchange 
{
	
    protected ServiceEndpoint endpoint;
    
    protected QName interfaceName;
    
    protected QName service;
    
    protected Exception error;
    
    protected Fault fault;
   
    protected HashMap<String,NormalizedMessage> messages = new HashMap<String,NormalizedMessage>();
    
    protected QName operation;
    
    protected URI pattern;
    
    protected HashMap<String,Object> properties = new HashMap<String,Object>();
    
    protected String exchangeId;
    
    protected Role role;
    
    protected DeliveryChannelImpl consumerDeliveryChannel;
    
    protected ExchangeStatus status;
    
    protected boolean transacted;
    
    protected boolean terminated = false;
    
    protected MessageExchangeDecorator consumerMEDecorator = null;
    
    protected MessageExchangeDecorator providerMEDecorator = null;
    
    ////////////////////////////////////////////////////////////
    //
    // implementation specific attributes
    //
    ////////////////////////////////////////////////////////////
    
    
    public static final String IN_MSG = "in";
    public static final String OUT_MSG = "out";
    
    public static URI IN_ONLY_PATTERN;
    public static URI IN_OPTIONAL_OUT_PATTERN;
    public static URI IN_OUT_PATTERN;
    public static URI ROBUST_IN_ONLY_PATTERN;
    
    static
    {
        try 
        {
            IN_ONLY_PATTERN = new URI("http://www.w3.org.2004/08/wsdl/in-only");
            
            IN_OPTIONAL_OUT_PATTERN = new URI("http://www.w3.org.2004/08/wsdl/in-opt-out");
            
            IN_OUT_PATTERN = new URI("http://www.w3.org.2004/08/wsdl/in-out");
            
            ROBUST_IN_ONLY_PATTERN = new URI("http://www.w3.org.2004/08/wsdl/robust-in-only");
        } 
        catch (URISyntaxException e) 
        {
            // Should never be thrown
            System.err.println("unexpected Exception in URI creation");
            e.printStackTrace(System.err);
        }
    }
    
/**
  * Create a MessageExchangeImpl
  * This constructor is called by a consumer
  * who wants to initialize an exchange.
 * @param consumerDeliveryChannel
 */
    public MessageExchangeImpl(DeliveryChannelImpl consumerDeliveryChannel)
    {	
        
        status = ExchangeStatus.ACTIVE;
        
        role = Role.CONSUMER;
        
        this.consumerDeliveryChannel = consumerDeliveryChannel;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#createFault()
     */
    public Fault createFault() throws MessagingException 
    {
        return new FaultImpl();
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#createMessage()
     */
    public NormalizedMessage createMessage() throws MessagingException 
    {
        return new NormalizedMessageImpl();
    }
    
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getEndpoint()
     */
    public ServiceEndpoint getEndpoint() 
    {
        return endpoint;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getError()
     */
    public Exception getError() 
    {
        return error;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getExchangeId()
     */
    public String getExchangeId() 
    {
        return exchangeId;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getFault()
     */
    public Fault getFault() 
    {
        return fault;
    }
    
    public QName getInterfaceName()
    {
        return interfaceName;
    }
    
    /**
     * get the message associated to the reference. 
     * It is <b>Not</b> Case Sensitive.
     * @see javax.jbi.messaging.MessageExchange#getMessage(java.lang.String)
     */
    public NormalizedMessage getMessage(String name) 
    {
    	if(name != null)
    		return (NormalizedMessage)messages.get(name.toLowerCase());
    	else 
    		return null;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getOperation()
     */
    public QName getOperation() 
    {
        return operation;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getPattern()
     */
    public URI getPattern() 
    {
        return pattern;
    }
    
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getProperty(java.lang.String)
     */
    public Object getProperty(String name) 
    {
        return properties.get(name);
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getService()
     */
    public QName getService() 
    {
        return service;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getPropertyNames()
     */
    public Set getPropertyNames()
    {
        return properties.keySet();
    }
    
    /**
     * (suivant que l on cr un nouveau ou qu'on rcupre de la couche de transport)
     * (non-Javadoc)
     * @see javax.jbi.messaging.MessageExchange#getRole()
     */
    public Role getRole()
    {
        return role;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getStatus()
     */
    public javax.jbi.messaging.ExchangeStatus getStatus() 
    {
        return status;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#isTransacted()
     */
    public boolean isTransacted() 
    {
        return transacted;
    }
      
    /**
     * @see javax.jbi.messaging.MessageExchange#setEndpoint(java.lang.Exception)
     */
    public void setEndpoint(ServiceEndpoint endpoint) 
    {
        this.endpoint = endpoint;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#setError(java.lang.Exception)
     */
    public void setError(Exception error) 
    {
        this.error = error;
        
        this.status = ExchangeStatus.ERROR;
    }
    
    /**
     * (non-Javadoc)
     * @see javax.jbi.messaging.MessageExchange#setFault(javax.jbi.messaging.Fault)
     */
    public void setFault(Fault fault) throws MessagingException 
    {
        this.fault = fault;
    }
    
    public void setInterfaceName(QName interfaceName)
    {
        this.interfaceName = interfaceName;
    }
    
    /**
     * Set the specified message.
     * Check the availability to set the message with the following rules:
     * <br>
     * <li>CONSUMER Role <-> in reference or PROVIDER Role <-> out reference</li>
     * <li>JBI predefined MEP <-> in or out reference</li>
     * <li>setMessage(XXX,reference) has not already been called.
     * <br>
     * This is <b>not</b> case sensitive
     * @see javax.jbi.messaging.MessageExchange#setMessage(javax.jbi.messaging.NormalizedMessage, java.lang.String)
     */
    public void setMessage(NormalizedMessage msg, String name)
    throws MessagingException 
    {
    	if(msg == null)
    		throw new MessagingException("NormalizedMessage must be non null");
    	if(name == null || name.trim().length()== 0)
    		throw new MessagingException("The message reference must be non null and non empty");
        
    	// check if the pattern allows this reference
    	checkPatternMatching(name);
    	
    	// check Role
    	checkRoleMatching(name);

    	// check that the message has not already been sent
    	if(messages.containsKey(name.toLowerCase()))
    		throw new MessagingException("A message has already been set with the '"+name+"' reference.");
    	
    	// no exception was thrown, set the message
    	messages.put(name.toLowerCase(),msg);
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#setOperation(javax.xml.namespace.QName)
     */
    public void setOperation(QName name) 
    {
        this.operation = name;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#setProperty(java.lang.String, java.lang.Object)
     */
    public void setProperty(String name, Object obj) 
    {
        properties.put(name,obj);
    }
    
    /**
     * set Destination ServiceName.
     * A destinsation Endpointreference is created with only "serviceName" attribute
     */
    public void setService(QName service) 
    {
        this.service = service;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#setStatus(javax.jbi.messaging.ExchangeStatus)
     */
    public void setStatus(ExchangeStatus status) throws MessagingException 
    {
        this.status = status;
    }
    
    
    /**
     * @param exchangeId The exchangeId to set.
     */
    public void setExchangeId(String exchangeId) {
        this.exchangeId = exchangeId;
    }
    /**
     * @param pattern The pattern to set.
     */
    public void setPattern(URI pattern) {
        this.pattern = pattern;
    }
    /**
     * @param transacted The transacted to set.
     */
    public void setTransacted(boolean transacted) {
        this.transacted = transacted;
    }
    
    
    
    public DeliveryChannelImpl getConsumerDeliveryChannel() {
		return consumerDeliveryChannel;
	}

	public void setRole(Role role)
    {
        this.role = role;
    }
	
	
	
	////////////////////////////////////////////
	// specific methods
	////////////////////////////////////////////
   
	public boolean isTerminated() {
		return terminated;
	}

	public void setTerminated(boolean terminated) {
		this.terminated = terminated;
	}

	public MessageExchangeDecorator getConsumerMEDecorator() {
		return consumerMEDecorator;
	}

	public void setConsumerMEDecorator(MessageExchangeDecorator consumerMEDecorator) {
		this.consumerMEDecorator = consumerMEDecorator;
	}

	
	public MessageExchangeDecorator getProviderMEDecorator() {
		return providerMEDecorator;
	}

	public void setProviderMEDecorator(MessageExchangeDecorator providerMEDecorator) {
		this.providerMEDecorator = providerMEDecorator;
	}

	/**
	 * Check that the exchange MEP allows the specified
	 * reference.
	 * This only works for the JBI predefined MEPs.
	 * It check that reference is IN or OUT.
	 * Other reference or MEP will cause an exception
	 * @param name
	 * @throws MessagingException the MEP does not allow the reference
	 */
	private void checkPatternMatching(String name) throws MessagingException
	{
		if(pattern == null)
			throw new MessagingException("the MEP is not defined");
		
		if(name == null)
			throw new MessagingException("the reference name is not defined");
		
		if(IN_MSG.equalsIgnoreCase(name))
		{
			if( IN_ONLY_PATTERN.equals(pattern)
			|| ROBUST_IN_ONLY_PATTERN.equals(pattern)
			|| IN_OUT_PATTERN.equals(pattern)
			|| IN_OPTIONAL_OUT_PATTERN.equals(pattern))
				return;
		}
		else if(OUT_MSG.equalsIgnoreCase(name))
		{
			if(IN_OUT_PATTERN.equals(pattern)
			|| IN_OPTIONAL_OUT_PATTERN.equals(pattern))
				return;
		}
		throw new MessagingException("the MEP does not allows this reference name");
	}
	
	/**
	 * Check that the Role allows the specified
	 * reference.
	 * This only works for the JBI predefined MEPs.
	 * It check that reference is IN or OUT and the Role
	 * is CONSUMER or PROVIDER.
	 * Other reference will cause an exception
	 * @param name
	 * @throws MessagingException the Role does not allow the reference
	 */
	private void checkRoleMatching(String name) throws MessagingException
	{
		if(role == null)
			throw new MessagingException("the Role is not defined");
		
		if(name == null)
			throw new MessagingException("the reference name is not defined");
		
		if(MessageExchange.Role.CONSUMER.equals(role))
		{
			if(IN_MSG.equalsIgnoreCase(name))
				return;
		}
		else if(MessageExchange.Role.PROVIDER.equals(role))
		{
			if(OUT_MSG.equalsIgnoreCase(name))
				return;
		}
		throw new MessagingException("the Role does not allows this reference name");
	}
	
    public String toString()
    {
        return "MessageExchange@"+getExchangeId();
    }

	
}
