/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: DeliveryChannelImpl.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.messaging;

import static org.objectweb.petals.util.InterfaceNamesHelper.LOGGER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_REGISTRY_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.JBI_ROUTER_NAME;

import javax.jbi.messaging.DeliveryChannel;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.jbi.component.ComponentContextImpl;
import org.objectweb.petals.jbi.registry.RegistryManager;
import org.objectweb.petals.jbi.routing.Router;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * TODO DeliveryChannelImpl:check multi thread use
 * @author Adrien LOUIS - EBM WebSourcing
 * @author Gael BLONDELLE - EBM WebSourcing
 */
public class MessagingServiceImpl implements MessagingService, BindingController, LifeCycleController
{
	private static final String[] CLIENT_INTERFACES = new String[] {
		JBI_REGISTRY_NAME, JBI_ROUTER_NAME };

	/**
	 * Monolog Logger instance.
	 */
	private Logger logger;

	/**
	 * The container router instance.
	 */
	private Router router;
	
	/**
	 * The container router instance.
	 */
	private RegistryManager registry;
	
	/**
	 * Component LifeCycle state.
	 */
	private String fcState;
  	
	public DeliveryChannel loadDeliveryChannel(ComponentContextImpl context)
	{
		LoggingUtil.start(logger);
    	
		DeliveryChannelImpl dc = new DeliveryChannelImpl(context,router,logger);
    	
    	context.setDeliveryChannel(dc);

    	LoggingUtil.end(logger);
    	
    	return dc;
    }
	
	public RegistryManager getRegistry()
	{
		return registry;
	}
    
	// ----------------------------------------------------------------------
	//   Fractal BindingController implementation
	// ----------------------------------------------------------------------


	/**
	 * @see org.objectweb.fractal.api.control.BindingController#listFc()
	 */
	public String[] listFc() {
		return CLIENT_INTERFACES;
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
	 */
	public Object lookupFc(String clientItfName)
			throws NoSuchInterfaceException {

		if (JBI_REGISTRY_NAME.equals(clientItfName)) {
			return registry;
		} 
		if (JBI_ROUTER_NAME.equals(clientItfName)) {
			return router;
		} 
		else {
			throw new NoSuchInterfaceException(clientItfName);
		}
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
	 */
	public void bindFc(String clientItfName, Object serverItf)
			throws NoSuchInterfaceException, IllegalBindingException,
			IllegalLifeCycleException {

		if (LOGGER_NAME.equals(clientItfName)) {
			logger = (Logger) serverItf;
		}
		else if (JBI_REGISTRY_NAME.equals(clientItfName)) {
			registry = (RegistryManager) serverItf;
		}
		else if (JBI_ROUTER_NAME.equals(clientItfName)) {
			router = (Router) serverItf;
		}
		else {
			throw new NoSuchInterfaceException(clientItfName);
		}
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
	 */
	public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
			IllegalBindingException, IllegalLifeCycleException {

		if (LOGGER_NAME.equals(clientItfName)) {
			logger = null;
		}
		else if (JBI_REGISTRY_NAME.equals(clientItfName)) {
			registry = null;
		}
		else if (JBI_ROUTER_NAME.equals(clientItfName)) {
			router = null;
		}
		else {
			throw new NoSuchInterfaceException(clientItfName);
		}
	}

	// ----------------------------------------------------------------------
	//   Fractal LifecycleController implementation
	// ----------------------------------------------------------------------

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
	 */
	public String getFcState() {
		return (STARTED.equals(fcState) ? STARTED : STOPPED);
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
	 */
	public void startFc() throws IllegalLifeCycleException 
	{
		fcState = STARTED;
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
	 */
	public void stopFc() throws IllegalLifeCycleException 
	{
		fcState = STOPPED;
	}

}
