/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NormalizedMessageImpl.java 76 2006-01-17 17:15:44Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.messaging;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.security.auth.Subject;
import javax.xml.transform.Source;

/**
 * JBI <code>NormalizedMessage</code> implementation.
 *
 * @version	$Rev: 76 $ $Date: 2006-01-17 18:15:44 +0100 (mar., 17 janv. 2006) $
 * @since	Petals 1.0
 * @author  Adrien LOUIS,
 *          <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class NormalizedMessageImpl implements NormalizedMessage {

	/**
	 * Attachments to the main message: id + content.
	 */
	private Map<String, DataHandler> attachments;

	/**
	 * The XML "payload" message.
	 */
	private Source content;

	/**
	 * Metadata associated with the message content. Map: property + value. 
	 */
	private Map<String, Object> properties;

	/**
	 * JAAS Security Subject.
	 */
	private Subject subject;

	/**
	 * This default constructor creates a new empty
	 * <code>NormalizedMessage</code>.
	 */
	public NormalizedMessageImpl() {
		super();

		content	= null;
		properties = new HashMap<String, Object>();
		attachments = new HashMap<String, DataHandler>();
	}

	/* ----------------------------------------------------------------------
	 * N o r m a l i z e d M e s s a g e    M e t h o d s
	 * ----------------------------------------------------------------------
	 */


	/**
	 * @inheritDoc
	 */
	public void addAttachment(String id, DataHandler content)
			throws MessagingException 
	{
		attachments.put(id, content);
	}

	/**
	 * @inheritDoc
	 */
	public DataHandler getAttachment(String id) 
	{
		return (DataHandler) attachments.get(id);
	}

	/**
	 * @inheritDoc
	 */
    public Set getAttachmentNames()
    {
        return attachments.keySet();
    }

	/**
	 * @inheritDoc
	 */
	public Source getContent() 
	{
		return content;
	}

	/**
	 * @inheritDoc
	 */
	public Object getProperty(String name) 
	{
		return properties.get(name);
	}

	/**
	 * @inheritDoc
	 */
	public Set getPropertyNames() 
	{
		return properties.keySet();
	}

	/**
	 * @inheritDoc
	 */
	public Subject getSecuritySubject() 
	{
		return subject;
	}

	/**
	 * @inheritDoc
	 */
	public void removeAttachment(String id) throws MessagingException 
	{
		Object att = attachments.remove(id);
		
		if(att == null)
		{
		    throw new MessagingException(id+" attachment does not exist.");
		}
	}

	/**
	 * @inheritDoc
	 */
	public void setContent(Source content) throws MessagingException 
	{
		this.content = content;
	}

	/**
	 * @inheritDoc
	 */
	public void setProperty(String name, Object value) 
	{
		if (value == null) {
			// case to remove properties not defined (null)
			properties.remove(name);

		} else {
			// define this property name with a new value
			properties.put(name,value);
		}
	}

	/**
	 * @inheritDoc
	 */
	public void setSecuritySubject(Subject subject) 
	{
		this.subject = subject;
	}

}
