/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: AbstractEndpoint.java 215 2006-04-13 13:53:51Z alouis $
 * -------------------------------------------------------------------------
 * $Log: AbstractEndpoint.java,v $ 
 */
package org.objectweb.petals.jbi.registry;

import java.io.ByteArrayInputStream;

import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.xml.sax.InputSource;

/**
 * @version	$Rev: 215 $ $Date: 2006-04-13 15:53:51 +0200 (jeu., 13 avr. 2006) $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>,
 * Adrien LOUIS
 */
public abstract class AbstractEndpoint implements ServiceEndpoint {

	private QName serviceName;
	private String endpointName;
	
	public static final String EPR_NAMESPACE = "http://java.sun.com/xml/ns/jbi";
	
	/**
	 * @param endpointName 
	 * @param serviceName 
	 * 
	 */
	public AbstractEndpoint(QName serviceName, String endpointName) {
		super();
		this.serviceName = serviceName;
		this.endpointName = endpointName;
	}

	/**
	 * @see javax.jbi.servicedesc.ServiceEndpoint#getAsReference(javax.xml.namespace.QName)
	 */
	public DocumentFragment getAsReference(QName operationName) 
	{
		DocumentFragment result = null;
		try 
		{
			String reference =
				"<end-point-reference xmlns='"+
				EPR_NAMESPACE+
				"' service-name = '>"+
				serviceName+
				"' end-point-name = '"+
				endpointName+
				"'></end-point-reference>";
			
			InputSource source = createSource(reference);
			
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			
			DocumentBuilder builder = factory.newDocumentBuilder();
			
			Document document = builder.parse(source);
			
			result = document.createDocumentFragment();
		} 
		catch (Exception e) 
		{
			e.printStackTrace();
		}  
		return result;
	}

	/**
	 * @see javax.jbi.servicedesc.ServiceEndpoint#getEndpointName()
	 */
	public String getEndpointName() {
		return endpointName;
	}

	/**
	 * @see javax.jbi.servicedesc.ServiceEndpoint#getInterfaces()
	 */
	public QName[] getInterfaces() {
		throw new NoSuchMethodError();
	}

	/**
	 * @see javax.jbi.servicedesc.ServiceEndpoint#getServiceName()
	 */
	public QName getServiceName() {
		return serviceName;
	}

	/**
	 * 
	 * @param interfaceName
	 * @return
	 */
	public boolean implementsInterface(QName interfaceName) {

		boolean implement = false;

		for (QName anInterface : getInterfaces()) {
			if (anInterface.equals(interfaceName)) {
				implement = true;
				break;
			}
		}

		return implement;
	}
	
	protected InputSource createSource(String msg)
	{
		InputSource source = new InputSource();
		
		byte[] msgByte = msg.getBytes();
		
		ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
		
		source.setByteStream(in);
		
		return source;
	}
	
	public String toString()
	{
		return serviceName+" ->"+endpointName;
	}
}
