/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: HashEndpointGroup.java 190 2006-04-07 16:56:51Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.registry;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

/**
 * @version	$Rev: 190 $ $Date: 2006-04-07 18:56:51 +0200 (ven., 07 avr. 2006) $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class HashEndpointGroup implements EndpointGroup {

	/**
	 * 
	 */
	private Map<String, AbstractEndpoint> endpoints;

	/**
	 * Default constructor.
	 */
	public HashEndpointGroup() {
		super();

		endpoints = new HashMap<String, AbstractEndpoint>();
	}

	/**
	 * @see org.objectweb.petals.jbi.registry.EndpointGroup#add(AbstractEndpoint)
	 */
	public void add(AbstractEndpoint endpoint) {

		String key = getKey(endpoint.getServiceName(),
				endpoint.getEndpointName());
		endpoints.put(key, endpoint);

	}

	/**
	 * @see org.objectweb.petals.jbi.registry.EndpointGroup#contains(javax.xml.namespace.QName, java.lang.String)
	 */
	public boolean contains(QName serviceName, String endpointName) {

		String key = getKey(serviceName, endpointName);
		return endpoints.containsKey(key);
	}

	/**
	 * @see org.objectweb.petals.jbi.registry.EndpointGroup#getEndpoint(javax.xml.namespace.QName, java.lang.String)
	 */
	public AbstractEndpoint getEndpoint(QName serviceName, String endpointName) {

		String key = getKey(serviceName, endpointName);
		return endpoints.get(key);
	}

	/**
	 * @see org.objectweb.petals.jbi.registry.EndpointGroup#getEndpointsForService(javax.xml.namespace.QName)
	 */
	public AbstractEndpoint[] getEndpointsForService(QName serviceName) {

		List<AbstractEndpoint> result = new ArrayList<AbstractEndpoint>();

		for (AbstractEndpoint anEndpoint : endpoints.values()) {

			if (anEndpoint.getServiceName().equals(serviceName)) {
				result.add(anEndpoint);
			}

		}

		return result.toArray(new AbstractEndpoint[result.size()]);
	}

	/**
	 * If interfaceName is null, return all endpoints
	 * @see org.objectweb.petals.jbi.registry.EndpointGroup#getEndpointsForInterface(javax.xml.namespace.QName)
	 */
	public AbstractEndpoint[] getEndpointsForInterface(QName interfaceName) {

		if(interfaceName != null)
		{
		List<AbstractEndpoint> result = new ArrayList<AbstractEndpoint>();

		for (AbstractEndpoint anEndpoint : endpoints.values()) {

			if (anEndpoint.implementsInterface(interfaceName)) {
				result.add(anEndpoint);
			}

		}
		return result.toArray(new AbstractEndpoint[result.size()]);
		}
		else
		{
			return endpoints.values().toArray(new AbstractEndpoint[0]);
		}
	}

	/**
	 * @see org.objectweb.petals.jbi.registry.EndpointGroup#remove(ServiceEndpoint)
	 */
	public void remove(ServiceEndpoint endpoint) {

		String key = getKey(endpoint.getServiceName(),
				endpoint.getEndpointName());

		endpoints.remove(key);

	}

	/**
	 * 
	 * @param serviceName
	 * @param endpointName
	 * @return
	 */
	private static String getKey(QName serviceName, String endpointName) {
		String compositeKey = serviceName.toString() + "$" + endpointName;
		return compositeKey;
	}
}
