/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: RegistryManagerImpl.java 216 2006-04-13 14:18:14Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.registry;

import static org.objectweb.petals.util.InterfaceNamesHelper.LOGGER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.MONOLOG_FACTORY_NAME;

import javax.jbi.JBIException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.jbi.component.ComponentContextImpl;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * @version	$Rev: 216 $ $Date: 2006-04-13 16:18:14 +0200 (jeu., 13 avr. 2006) $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class RegistryManagerImpl implements RegistryManager, LifeCycleController, BindingController {

	private EndpointGroup internalEndpoints;

	private EndpointGroup externalEndpoints;
	
	/**
	 * Fractal Component Current State.
	 */
	private String state;

	/**
	 * Monolog Logger instance.
	 */
	private Logger logger;
	
	/**
	 * Logger wrapper
	 */
	private LoggingUtil log;
	
	/**
	 * Default constructor. 
	 */
	public RegistryManagerImpl() {
		super();
		internalEndpoints = new HashEndpointGroup();
		externalEndpoints = new HashEndpointGroup();
	}

	// ----------------------------------------------------------------------
	//   RegistryManager implementation
	// ----------------------------------------------------------------------

	/**
	 * @see org.objectweb.petals.jbi.registry.RegistryManager#deregisterInternalEndpoint(javax.jbi.servicedesc.ServiceEndpoint)
	 */
	public void deregisterInternalEndpoint(ServiceEndpoint endpoint)
			throws JBIException 
	{
		log.call(endpoint);
		
		internalEndpoints.remove(endpoint);
	}

	/**
	 * @see org.objectweb.petals.jbi.registry.RegistryManager#deregisterExternalEndpoint(javax.jbi.servicedesc.ServiceEndpoint)
	 */
	public void deregisterExternalEndpoint(ServiceEndpoint externalEndpoint)
			throws JBIException 
	{
		log.call(externalEndpoint);
		
		externalEndpoints.remove(externalEndpoint);
	}

	/**
	 * @see org.objectweb.petals.jbi.registry.RegistryManager#getInternalEndpoint(javax.xml.namespace.QName, java.lang.String)
	 */
	public ServiceEndpoint getInternalEndpoint(QName serviceName,
			String endpointName) 
	{
		log.call(serviceName+", "+endpointName);
		
		ServiceEndpoint se = internalEndpoints.getEndpoint(serviceName, endpointName);
		
		return se;
	}

	/**
	 * @see org.objectweb.petals.jbi.registry.RegistryManager#getInternalEndpointsForInterface(javax.xml.namespace.QName)
	 */
	public ServiceEndpoint[] getInternalEndpointsForInterface(
			QName interfaceName) 
	{
		log.call(interfaceName);
		
		ServiceEndpoint[] se = internalEndpoints.getEndpointsForInterface(interfaceName);
		
		return se;
	}

	/**
	 * @see org.objectweb.petals.jbi.registry.RegistryManager#getInternalEndpointsForService(javax.xml.namespace.QName)
	 */
	public ServiceEndpoint[] getInternalEndpointsForService(QName serviceName) 
	{
		log.call(serviceName);
		
		ServiceEndpoint[] se = internalEndpoints.getEndpointsForService(serviceName);
		
		return se;
	}

	/**
	 * @see org.objectweb.petals.jbi.registry.RegistryManager#getExternalEndpointsForInterface(javax.xml.namespace.QName)
	 */
	public ServiceEndpoint[] getExternalEndpointsForInterface(
			QName interfaceName) 
	{
		log.call(interfaceName);
		
		ServiceEndpoint[] se =  externalEndpoints.getEndpointsForInterface(interfaceName);
		
		return se;
	}

	/**
	 * @see org.objectweb.petals.jbi.registry.RegistryManager#getExternalEndpointsForService(javax.xml.namespace.QName)
	 */
	public ServiceEndpoint[] getExternalEndpointsForService(QName serviceName) 
	{
		log.call(serviceName);
		
		ServiceEndpoint[] se =  externalEndpoints.getEndpointsForService(serviceName);
		
		return se;
	}

	/**
	 * @throw JBIException serviceName or endpointName null or empty
	 * @see org.objectweb.petals.jbi.registry.RegistryManager#registerInternalEndpoint(javax.xml.namespace.QName, java.lang.String)
	 */
	public ServiceEndpoint registerInternalEndpoint(QName serviceName,
			String endpointName, ComponentContextImpl ctx) throws JBIException 
	{
		log.start(serviceName+", "+endpointName+", "+ctx);
		
		if(serviceName == null || endpointName == null || endpointName.trim().length()==0)
		{
			JBIException e = new JBIException("The specified service name is null");
			log.warning(e.getMessage(),e);
			throw e;
		}
		else if(endpointName == null || endpointName.trim().length()==0)
		{
			JBIException e = new JBIException("The specified endpoint name is null");
			log.warning(e.getMessage(),e);
			throw e;
		}
		
		AbstractEndpoint ep;

		// void duplicate endpoints registration
		if (internalEndpoints.contains(serviceName, endpointName)) 
		{
			ep = internalEndpoints.getEndpoint(serviceName, endpointName);
		} 
		else 
		{
			ep = new InternalEndpoint(serviceName, endpointName,ctx);
		
			internalEndpoints.add(ep);
		}
		log.end();
		
		return ep;
	}

	/**
	 * @see org.objectweb.petals.jbi.registry.RegistryManager#registerExternalEndpoint(javax.jbi.servicedesc.ServiceEndpoint)
	 */
	public void registerExternalEndpoint(ServiceEndpoint endpoint)
			throws JBIException 
	{
		log.start(endpoint);
		
		if(endpoint == null)
		{
			JBIException e = new JBIException("The specified endpoint is null");
			log.warning(e.getMessage(),e);
			throw e;
		}
		
		QName serviceName = endpoint.getServiceName();
		
		String endpointName = endpoint.getEndpointName();

		// void duplicate endpoints registration
		if (!externalEndpoints.contains(serviceName, endpointName)) 
		{
			AbstractEndpoint ep = new ExternalEndpoint(endpoint);
			
			externalEndpoints.add(ep);
		}
		else
		{
			JBIException e = new JBIException("The specified endpoint already exists");
			log.warning(e.getMessage(),e);
			throw e;
		}
		log.end();
	}
	
	
	// ----------------------------------------------------------------------
	//   Fractal BindingController implementation
	// ----------------------------------------------------------------------

	private static final String[] CLIENT_INTERFACES = new String[0];
	
	/**
	 * @see org.objectweb.fractal.api.control.BindingController#listFc()
	 */
	public String[] listFc() 
	{
		return CLIENT_INTERFACES;
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
	 */
	public Object lookupFc(String clientItfName)
			throws NoSuchInterfaceException 
	{
			throw new NoSuchInterfaceException(clientItfName);
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
	 */
	public void bindFc(String clientItfName, Object serverItf)
			throws NoSuchInterfaceException, IllegalBindingException,
			IllegalLifeCycleException {

		if (LOGGER_NAME.equals(clientItfName)) {
			logger = (Logger) serverItf;
		} else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
				//nothing
		} else{
			throw new NoSuchInterfaceException(clientItfName);
		}
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
	 */
	public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
			IllegalBindingException, IllegalLifeCycleException {

		if (LOGGER_NAME.equals(clientItfName)) {
			logger = null;
		}
		else {
			throw new NoSuchInterfaceException(clientItfName);
		}
	}


	// ---------------------------------------------------------------------
	//   Fractal LifeCycleController implementation
	// ----------------------------------------------------------------------

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
	 */
	public String getFcState() {
		return (state != null ? state : STOPPED);
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
	 */
	public void startFc() throws IllegalLifeCycleException {

		state = STARTED;
		
		log = new LoggingUtil(logger);
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
	 */
	public void stopFc() throws IllegalLifeCycleException {

		state = STOPPED;
	}

}
