/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: TransporterImpl.java,v 0.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.transport;

import static org.objectweb.petals.util.InterfaceNamesHelper.LOGGER_NAME;
import static org.objectweb.petals.util.InterfaceNamesHelper.MONOLOG_FACTORY_NAME;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.petals.jbi.messaging.DeliveryChannelImpl;
import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.jbi.registry.InternalEndpoint;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

public class TransporterImpl implements Transporter,BindingController, LifeCycleController {

	
	private static final String[] CLIENT_INTERFACES = new String[] {};

	
	private LoggingUtil log;
	
	/**
	 * Component LifeCycle state.
	 */
	private String fcState;
	
	/**
	 * send the exchange to the consumer, set the Role to 'consumer' 
	 */
	public void sendToConsumer(MessageExchangeImpl exchange) throws TransportException 
	{
		log.start();
		
		//get the consumer' deliverychannel
    	
		DeliveryChannelImpl consumerDeliveryChannel = exchange.getConsumerDeliveryChannel();
    	
    	try 
    	{
			exchange.setRole(MessageExchange.Role.CONSUMER);
			
			consumerDeliveryChannel.push(exchange);
		} 
    	catch (JBIException e) 
    	{
    		throw new TransportException(e);
		}
    	log.end();
	}

	/**
	 * send the exchange to the provider, set the Role to 'provider' 
	 */
	public void sendToProvider(MessageExchangeImpl exchange) throws TransportException 
	{
		log.start();

		ServiceEndpoint ep = exchange.getEndpoint();
    	
    	// Get the destination Component context
    	ComponentContext ctx = ((InternalEndpoint)ep).getComponentContext();
    	
    	try 
    	{
			exchange.setRole(MessageExchange.Role.PROVIDER);
			
			((DeliveryChannelImpl)ctx.getDeliveryChannel()).push(exchange);
		} 
    	catch (JBIException e) 
    	{
    		throw new TransportException(e);
		}
    	log.end();
	}
    
    
    
    //  ----------------------------------------------------------------------
	//   Fractal BindingController implementation
	// ----------------------------------------------------------------------


	

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#listFc()
	 */
	public String[] listFc() {
		return CLIENT_INTERFACES;
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
	 */
	public Object lookupFc(String clientItfName)
			throws NoSuchInterfaceException {
		throw new NoSuchInterfaceException(clientItfName);		
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
	 */
	public void bindFc(String clientItfName, Object serverItf)
			throws NoSuchInterfaceException, IllegalBindingException,
			IllegalLifeCycleException {

		if (LOGGER_NAME.equals(clientItfName)) {
			log = new LoggingUtil((Logger) serverItf);
		}else if (MONOLOG_FACTORY_NAME.equals(clientItfName)) {
			//nothing
		}else {
			throw new NoSuchInterfaceException(clientItfName);
		}
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
	 */
	public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
			IllegalBindingException, IllegalLifeCycleException {
		throw new NoSuchInterfaceException(clientItfName);
	}

	// ----------------------------------------------------------------------
	//   Fractal LifecycleController implementation
	// ----------------------------------------------------------------------

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
	 */
	public String getFcState() {
		return (STARTED.equals(fcState) ? STARTED : STOPPED);
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
	 */
	public void startFc() throws IllegalLifeCycleException {

    	fcState = STARTED;
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
	 */
	public void stopFc() throws IllegalLifeCycleException {

		fcState = STOPPED;
	}
}
