/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JBIDescriptorBuilderTest.java 218 2006-04-13 14:26:43Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.descriptor;

import java.io.File;
import java.net.URI;
import java.net.URL;

import junit.framework.TestCase;

import org.objectweb.petals.jbi.descriptor.JBIDescriptorBuilder;

/**
 * @version	$Rev: 218 $ $Date: 2006-04-13 16:26:43 +0200 (jeu., 13 avr. 2006) $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class JBIDescriptorBuilderTest extends TestCase {

	/**
	 * Testing load and parse operation on descriptors files listed in
	 * testing resource data directory.
	 *
	 * @throws Exception
	 */
	public final void testLoadingDescriptors() throws Exception {

		File[] descriptors = listDescritorsTestingData();

		if (descriptors != null) {
			for (File aFile : descriptors) {
				
				new JBIDescriptorBuilder(aFile.toURI(),null).parseJbiXml( aFile.toURI());
			}
		}
	}

	/**
	 * Testing validation using the JBI Descriptor XML Schema on descriptors
	 * files listed in testing resource data directory.
	 * 
	 * @throws Exception
	 */
	public final void testValidatingDescriptors() throws Exception {

		File[] descriptors = listDescritorsTestingData();

		if (descriptors != null) {
			for (File aFile : descriptors) {
	
				URI sourceURI = aFile.toURI();
				assertTrue( new JBIDescriptorBuilder(sourceURI,null).validateJbiXml(sourceURI) );
			}
		}
	}

	/**
	 * @return
	 * @throws Exception 
	 */
	private File[] listDescritorsTestingData() throws Exception {

		ClassLoader cl = Thread.currentThread().getContextClassLoader();

		URL markerURL;
		if ((markerURL = cl.getResource("data-marker")) == null) {
			return null;
		}

		String baseDir = markerURL.toURI()
			.resolve(".")
			.getPath();

		File dir = new File(baseDir + File.separator + "descriptors");

		if (dir == null || !dir.exists()) {
			return null;
		}

		return dir.listFiles();
	}
}
