/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JBIPackageTest.java 171 2006-04-03 09:36:08Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.objectweb.petals.jbi.management.packaging.JBIPackage;
import org.objectweb.petals.jbi.management.packaging.JBIPackageBuilder;

import junit.framework.TestCase;

/**
 * @version	$Rev: 171 $ $Date: 2006-04-03 11:36:08 +0200 (lun., 03 avr. 2006) $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class JBIPackageTest extends TestCase {

	/**
	 * List all packages contained in package testing resource
	 * (test-data/packages) and build the JBIPackage asserting it's not null.
	 */
	public void testBuildingTestJbiPackages() throws Exception {

		URL[] packageList = listPackagesTestingData();

		for (URL anUrl : packageList) {

			JBIPackageBuilder builder = new JBIPackageBuilder(anUrl.toString(),null);
			JBIPackage pkg = builder.build();

			assertNotNull(pkg);
		}
	}

	/**
	 * @return
	 * @throws Exception 
	 */
	private URL[] listPackagesTestingData() throws Exception {

		List<URL> result = new ArrayList<URL>();

		ClassLoader cl = Thread.currentThread().getContextClassLoader();

		URL markerURL;
		if ((markerURL = cl.getResource("data-marker")) == null) {
			return null;
		}

		String baseDir = markerURL.toURI()
			.resolve(".")
			.getPath();

		File dir = new File(baseDir + File.separator + "packages");

		if (dir == null || !dir.exists()) {
			return null;
		}

		for (File aFile : dir.listFiles()) {
			result.add(aFile.toURL());
		}

		return result.toArray(new URL[result.size()]);
	}


}
