/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: FractalKernel.java 76 2006-01-17 17:15:44Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.fractal;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.util.monolog.api.LoggerFactory;

import org.objectweb.petals.kernel.Kernel;

/**
 * 
 * @version $Rev: 76 $ $Date: 2006-01-17 18:15:44 +0100 (mar., 17 janv. 2006) $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class FractalKernel implements Kernel, BindingController,
		LifeCycleController {

	private static String LOGGING_INTERFACE_NAME = "logging";

	private static String[] CLIENT_INTERFACES = new String[] {
		LOGGING_INTERFACE_NAME
	};

	private LoggerFactory logging;

	/**
	 * Fractal Component state.
	 */
	private String fcState;

	public FractalKernel() {
		super();
	}

	// ----------------------------------------------------------------------
	// Kernel interface implementation
	// ----------------------------------------------------------------------

	/**
	 * @see org.objectweb.petals.kernel.Kernel#boot()
	 */
	public void boot() {
		// TODO Auto-generated method stub
	}

	/**
	 * @see org.objectweb.petals.kernel.Kernel#isRunning()
	 */
	public boolean isRunning() {
		// TODO Auto-generated method stub
		return false;
	}

	/**
	 * @see org.objectweb.petals.kernel.Kernel#shutdown()
	 */
	public void shutdown() {
		// TODO Auto-generated method stub
	}

	// ----------------------------------------------------------------------
	// Fractal BindingController interface implementation
	// ----------------------------------------------------------------------

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#listFc()
	 */
	public String[] listFc() {
		return CLIENT_INTERFACES;
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
	 */
	public Object lookupFc(String clientItfName)
			throws NoSuchInterfaceException {

		if (LOGGING_INTERFACE_NAME.equals(clientItfName)) {
			return logging;
		} else {
			throw new NoSuchInterfaceException(clientItfName);
		}
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
	 *      java.lang.Object)
	 */
	@SuppressWarnings("unchecked")
	public void bindFc(String clientItfName, Object serverItf)
			throws NoSuchInterfaceException, IllegalBindingException,
			IllegalLifeCycleException {

		if (LOGGING_INTERFACE_NAME.equals(clientItfName)) {
			logging = (LoggerFactory) serverItf;
		} else {
			throw new NoSuchInterfaceException(clientItfName);
		}
	}

	/**
	 * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
	 */
	public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
			IllegalBindingException, IllegalLifeCycleException {

		if (LOGGING_INTERFACE_NAME.equals(clientItfName)) {
			logging = null;
		} else {
			throw new NoSuchInterfaceException(clientItfName);
		}
	}

	// ----------------------------------------------------------------------
	// Fractal LifeCycleController interface implementation
	// ----------------------------------------------------------------------

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
	 */
	public String getFcState() {
		return (fcState == STARTED ? STARTED : STOPPED);
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
	 */
	public void startFc() throws IllegalLifeCycleException {
		fcState = STARTED;
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
	 */
	public void stopFc() throws IllegalLifeCycleException {
		fcState = STOPPED;
	}

}
