/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MX4J_HtmlAdaptor.java 76 2006-01-17 17:15:44Z rmarins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.fractal.mx4j;

import javax.management.Attribute;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import mx4j.tools.adaptor.http.HttpAdaptor;
import mx4j.tools.adaptor.http.HttpAdaptorMBean;
import mx4j.tools.adaptor.http.XSLTProcessor;

import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * This is the Fractal JMX adaptor for use with MX4J tools.
 *
 * @version	$Rev: 76 $ $Date: 2006-01-17 18:15:44 +0100 (mar., 17 janv. 2006) $
 * @since	Petals 1.0
 * @author	<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class MX4J_HtmlAdaptor implements BindingController,
		LifeCycleController, MX4JCommunicatorAttributes {
    
	private static final String ADAPTOR = "MX4J_HtmlAdaptor";

	private final static String BINDING_ADMIN_ATT = "adminAtt";

	private AdminAttributes adminAtt;

	private HttpAdaptorMBean adaptor = null;

	private int port;

	private String host;

	private Logger logger;

	private ObjectName oName;

	// --------------------------------------------------------------------------
	//   Fractal BindingController interface implementation
	// --------------------------------------------------------------------------
	public String[] listFc() {
		return new String[] {BINDING_ADMIN_ATT};
	}

	public Object lookupFc(final String itfName) {
		if (BINDING_ADMIN_ATT.equals(itfName)) {
			return adminAtt;
		} else {
			return null;
		}
	}

	public void bindFc(final String itfName, final Object itfValue) {
	    if ("logger".equals(itfName)) {
	        logger = (Logger) itfValue;
	    } else if (BINDING_ADMIN_ATT.equals(itfName)) {
			adminAtt = (AdminAttributes) itfValue;
		}
	}

	public void unbindFc(final String itfName) {
		if (BINDING_ADMIN_ATT.equals(itfName)) {
			adminAtt = null;
		}
	}
	
	// -------------------------------------------------------------------------
	//   Fractal JMX Communicator Attributes controller implementation
	// -------------------------------------------------------------------------

	public int getPort() {
		return port;
	}

	public void setPort(final int port) {
		this.port = port;
	}

	// -------------------------------------------------------------------------
	//   MX4JCommunicatorAttributes implementation
	// -------------------------------------------------------------------------

	public String getHost() {
		return host;
	}
	public void setHost(String host) {
		this.host = host;
	}

	// -------------------------------------------------------------------------
	//   Fractal LifeCycleController interface implementation
	// -------------------------------------------------------------------------

	public String getFcState() {
		return null;
	}

	public synchronized void startFc() {
		try {
			if (adaptor != null
				&& adaptor.isActive()
				&& adaptor.getPort() == port) {
				return;
			}

			// unregister & stops
			if (adaptor != null) {
				adaptor.stop();

				try {
					adminAtt.getRawMBeanServer().unregisterMBean(
							getObjectName());
				} catch (InstanceNotFoundException ignore) {}
			}

			// registers & starts

//			Log.redirectTo(new MX4JLoggerMonolog(logger));
			adaptor = new HttpAdaptor();

			adaptor.setPort(port);
			adaptor.setHost(host);

			MBeanServer server = adminAtt.getRawMBeanServer(); 
			server.registerMBean(adaptor, getObjectName());

			ObjectName processorName = new ObjectName(
					"Server:name=XSLTProcessor");
			XSLTProcessor processor = new XSLTProcessor();

			processor.setPathInJar(
					"org/objectweb/petals/kernel/fractal/mx4j/xsl");

			processor.setUseCache(false);
			server.registerMBean(processor, processorName);

			server.setAttribute(getObjectName(), 
			        new Attribute("ProcessorName", processorName));

			adaptor.start();

//			logger.log(BasicLevel.INFO, "MX4J HTTP Adaptor launched: " 
//			        + "\n\tUrl: http://" + adapter.getHost() 
//			        + ":" + adapter.getPort());
		} catch (Exception e) {
//		    logger.log(BasicLevel.ERROR, 
//		            "Error during the instanciation of the MX4J Htttp adaptor:", e);
		}
	}

	public void stopFc() {
		if (adaptor != null) {
		    adaptor.stop();
			try {
				adminAtt.getRawMBeanServer().unregisterMBean(getObjectName());
			} catch (Exception ignore) {
                if (logger != null) {
                    logger.log(BasicLevel.WARN, 
                            "Error during MBean unregistering: " 
                            + ignore.getMessage(), ignore);
                }
			}
		}
    }

	// -------------------------------------------------------------------------
	//   Private implementation methods
	// -------------------------------------------------------------------------

	private ObjectName getObjectName() throws MalformedObjectNameException {
		if (oName == null) {
			String id = '@' + Integer.toHexString(this.hashCode());
			oName = new ObjectName(ADAPTOR + id + ":type=html,port="
					+ adaptor.getPort());
		}
		return oName;
	}
}
